##############################################
#           Northern Quoll Genomics          #
##############################################

# Clear environment and set up working dir and datestamp
rm(list = ls())
setwd("E:/nor_quoll_genetics")
currentDate <- Sys.Date() # Add current date to environment for output of file names with date
options(scipen = 999) #prevent display in scientific notation


# Read in data and check
df <- read.csv("samp_submission/sampinfo250221.csv",na.strings=c(""," ","NA"))
str(df)
colnames(df)
head(df)
df$lat <- as.numeric(df$lat)
df$long <- as.numeric(df$long)
df$date <- format(as.Date(df$date, format="%d/%m/%Y"),"%Y-%m-%d")
df$popn <- as.factor(df$popn)
min(df$date,na.rm = TRUE) # oldest sample
max(df$date,na.rm = TRUE) # youngest sample
str(df)



### Map all samples used

# Load mapping packages
if (!require("ggplot2")) install.packages("ggplot2")
if (!require("mapdata")) install.packages("mapdata")
if (!require("ggsn")) install.packages("ggsn")

## Set up limits of maps to save time
# Need to replace missing values of lat long with the popn mean
df[5:6] <- sapply(df[5:6], function(x){
  ave(x, df[[4]], FUN = function(.x) {
    .x[is.na(.x)] <- mean(.x, na.rm = TRUE)
    .x
  })
})

mapdf <- df[!is.na(df$lat),] # In case there's any missing vals left..
xmin <- floor(min(mapdf$long))-1
xmax <- ceiling(max(mapdf$long))+1
ymin <- floor(min(mapdf$lat))-1.5
ymax <- ceiling(max(mapdf$lat))+1

# Create basemap
w2hr <- map_data("world2Hires", xlim = c(xmin,xmax), ylim = c(ymin,ymax), lforce="e") # Featureless

# Obtain mean values for lat and long
meanlatlong <- aggregate(mapdf[, 5:6], list(mapdf$popn), mean)

# Count number of rows/samples per population
library(plyr)
numsamps <- count(mapdf$popn)
colnames(numsamps) <- c("Group.1","freq")

# Combine dfs
sum.sites <- merge(numsamps,meanlatlong,by="Group.1")
colnames(sum.sites) <- c("population","sampsize","latitude","longitude")

# Adjust scalebar and north arrow scale as needed
quollmap <- ggplot() +
  geom_polygon(data = w2hr, aes(x=long, y = lat, group = group), fill = "grey85", color = "black") + 
  coord_fixed(1) +
  coord_equal(expand=FALSE) +
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid=element_blank()) +
  xlab("Longitude") +
  ylab("Latitude") +
  theme(axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 14, color="black")) +
  geom_point(data = sum.sites, aes(x = longitude, y = latitude), color = "black",
             size = sqrt(sum.sites$sampsize)*1.25, pch=21, fill="dodgerblue2") +
  north(location = "topleft", scale = 0.18, symbol = 1,
        y.min=ymin, y.max=ymax, x.min=xmin+15, x.max=xmax,
        anchor = c(x=xmin, y=ymax-(2/100*(xmax-xmin)))) +
  scale_y_continuous(breaks=c(-23, -19, -15, -11)) +
  scale_x_continuous(breaks=c(116,126,136,146)) +
  ggsn::scalebar(location="bottomleft",y.min=ymin, y.max=ymax, x.min=xmin, x.max=xmax, 
                 dist=500, dist_unit="km", transform=TRUE, model='WGS84',
                 anchor = c(x = xmax-11.5, y = ymin+0.8), st.color="black", st.dist=-0.05, st.size=4)

quollmap


## Output plots of species by year
pdffilename <- paste("output/quolls_map",currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=9, height=5.5)
quollmap
dev.off()





## Get maximum pairwise distance between samples

# get distance matrix of all samples
library("geodist")
distmatrix <- geodist(mapdf[ , c(5:6)], measure="geodesic")
distmatrix <- as.matrix(distmatrix)
dimnames(distmatrix) <- list(mapdf$sample)
paste("Average pairwise distance between all samples is", round(mean(distmatrix))/1000,"km")
paste("Maximum pairwise distance between all samples is", round(max(distmatrix))/1000,"km")
paste("Minimum pairwise distance between all samples is", round(min(distmatrix))/1000,"km")

# Retain only individuals that are really far apart and check population names
dimnames(distmatrix) <- list(mapdf$popn)
distmatrix <- round(distmatrix/1000)
distdf <- as.data.frame(distmatrix)
colnames(distdf) <- rownames(distdf)
distdf <- distdf[ , sapply(distdf, max) > 3222]
distdf <- distdf[distdf[1]>3222,]
distdf




################################
# Depth checking after angsd run
################################

# Sort out a global depth count file
library(data.table)
depth <- fread("output/dasy_280221.depthGlobal", select = c(1:70000))
# depth <- read.table("output/dasy_280221.depthGlobal")
depth <- as.data.frame(t(depth))
depth[1:10,]
colnames(depth) <- "nosites"
rownames(depth) <- seq(1:nrow(depth))
depth$readspersite <- as.numeric(rownames(depth))-1
depth$noreads <- depth$nosites*depth$readspersite
depth[1:10,1:3]
depth[800:810,1:3]
nrow(depth)
#depth <- depth[1:100000,] # Not necessary to remove the final row
hist(depth$noreads,ylim=c(0,8000),xlim=c(0,600000),breaks=50)
meancoverage <- sum(depth$noreads)/sum(depth$nosites)/475 # num samps
meancoverage


# Now use a loop to create a df with a row for every site, with number of reads in the adjacent column
bysite <- as.data.frame(rep(NA, sum(depth$nosites)))
colnames(bysite) <- "site"
bysite$site <- seq(1,nrow(bysite),1)
bysite$noreads <- NA
bysite[1:10,]

depth <- depth[depth$nosites>0,]
sitelist <- list()
i <- 1
while(i <= nrow(depth)) {
  sitelist[[i]] <- rep(depth$readspersite[i],depth$nosites[i])
  i <- i + 1
}
sitelist[1:10]

singlevec <- Reduce(c,sitelist)

bysite$noreads <- singlevec
bysite[1:10,]

hist(bysite$noreads, breaks=50)

sum(bysite$noreads) # Sum of reads per SNP (not really meaningful)
mean(bysite$noreads)/475 # This is mean reads/site/sample
median(bysite$noreads)/475 # This is median reads/site/sample









#############################
## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()




################################
## ALL SAMPLES INITIAL FILTER ##
################################

## Filter to good SNPs and samples dataset
# Read in snp sample matrix from ANGSD
dasygens <- read.table(file="output/dasy_280221.geno", header=FALSE, row.names = NULL, na.strings = -1) # 475 samples (incl replicates)

# Sort out formatting
dasygens$V1 <- paste(dasygens$V1,"-",dasygens$V2, sep="")
dasygens$V2 <- NULL
rownames(dasygens) <- dasygens$V1
dasygens$V1 <- NULL

samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL) # 475 samples
colnames(dasygens) <- samps$sample
dasygens[1:8,1:8]



##############################################################################
### Iteratively remove samples,snps,samples,snps,samples (O'Leary et al. 2018)

## Remove some really crappy samples before filtering loci
dasygens <- as.data.frame(t(dasygens))
nrow(dasygens) # number of samples before filter
hist(rowSums(is.na(dasygens)))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.995*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Filter out really crappy SNPs, based on callrate
# Manually calculate callrate
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.88,]

## Remove more crap samples samples before filtering loci again
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.96*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.91,]

# Remove more samples
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.85*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.92,]

# Remove more samples
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.75*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.94,]

# Remove more samples
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.6*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.95,]

# Remove more samples
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.5*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.96,]

# Remove more samples
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.4*ncol(dasygens), ] # Unavoidable loss of samples due to missingness

## Remove more SNPs
dasygens <- as.data.frame(t(dasygens))
dasygens$missingperloc <- rowSums(is.na(dasygens))
dasygens$samplestyped <- ncol(dasygens[1:(ncol(dasygens)-1)])-dasygens$missingperloc
dasygens$callrate <- dasygens$samplestyped/(dasygens$samplestyped+dasygens$missingperloc)
dasygens <- dasygens[dasygens$callrate > 0.97,]

# Remove more samples
dasygens <- as.data.frame(t(dasygens))
dasygens <- dasygens[rowSums(is.na(dasygens)) < 0.2*ncol(dasygens), ] # Unavoidable loss of samples due to missingness





#####################################
#### Filter on Minor Allele Frequency
# Can filter on the number of times a minor allele is present rather than the MAF (Roesti et al. 2012)
dasygens <- as.data.frame(t(dasygens))
dasygens$allelecount <- rowSums(dasygens[1:(ncol(dasygens)-2)],na.rm=TRUE)
dasygens$alf <- dasygens$allelecount/(2*dasygens$samplestyped)
dasygens$alleleA <- 1-dasygens$alf
dasygens$alf <- ifelse(dasygens$alf>0.5, dasygens$alleleA, dasygens$alf)
dasygens$alleleA <- 1-dasygens$alf
dasygens <- dasygens[dasygens$allelecount>=3,]


# Calculate Hexp and Hobs and only keep all loci with Ho <=0.x
# Hexp = 2pq = 1 - (p^2 + q^2)
dasygens$Hs <- 2*dasygens$alf*dasygens$alleleA
dasygens$Ho <- as.numeric(rowSums(dasygens[1:(ncol(dasygens)-7)]==1,na.rm=TRUE))
dasygens$Ho <- dasygens$Ho/dasygens$samplestyped

# Can check the influence of FIS filter on Hobs and MAF patterns
# FIS = 1 - (HO/HS)
# dasygens$fis <- 1-(dasygens$Ho/dasygens$Hs)
# dasygens <- dasygens[dasygens$fis<0.5,]

# Drop if Hobs is > 0.x
dasygens <- dasygens[dasygens$Ho<0.6,]


# Check patterns in SNPs
plot(dasygens$alf)
hist(dasygens$alf)
plot(dasygens$Hs,dasygens$Ho)
hist(dasygens$Ho)


# Remove extraneous columns
head(dasygens[,(ncol(dasygens)-9):(ncol(dasygens))]) # shows head of last x columns
dasygens <- dasygens[,1:(ncol(dasygens)-8)] # Removes last x columns
head(dasygens[,(ncol(dasygens)-9):(ncol(dasygens))]) # All good




# Change name so we don't have to keep filtering again and again and again and again and again
dasgen <- dasygens




######################
#### Filter on linkage
## Create new df for filtering
dasgen.linkage <- dasgen
dasgen.linkage[1:6,1:6]
dasgen.linkage$locus <- rownames(dasgen.linkage)

allloci <- read.csv("output/all_loci_dasy_280221.csv", header=TRUE)
dasgen.linkage <- merge(dasgen.linkage, allloci, all.x=TRUE, by="locus")
head(dasgen.linkage[ , (ncol(dasgen.linkage)-4) : (ncol(dasgen.linkage))]) # Check last columns

# source("https://bioconductor.org/biocLite.R")
# biocLite("SNPRelate")
library("gdsfmt")
library(SNPRelate)
library("ramify")

## Sort out a gds file for SNPrelate
genmat <- dasgen.linkage[, 2:(ncol(dasgen.linkage)-2)] # create SNP only matrix (i.e. remove chrom columns)
genmat[] <- lapply(genmat, function(x) {
  if(is.factor(x)) as.numeric(as.character(x)) else x
})
sapply(genmat, class)
genmat <- as.matrix(genmat)
SampID <- colnames(dasgen.linkage)[2:(ncol(dasgen.linkage)-2)] # Get a list of samples
SNP_ID <- dasgen.linkage[, 1] # Get a list of SNPs
Chrom <- as.integer(as.factor(dasgen.linkage[, ncol(dasgen.linkage)-1])) # Get a list of chromosomes corresponding to SNPs
ChromPos <- dasgen.linkage[, ncol(dasgen.linkage)] # Get a list of chromosome positions corresponding to SNPs

# Make a GDS dataset from the SNP x Sample matrix
gds <- snpgdsCreateGeno("output/LinkFilt010321.gds", genmat = genmat, sample.id = SampID, snp.id = SNP_ID, snp.chromosome = Chrom, snp.position = ChromPos, snpfirstdim = TRUE)
gds <- snpgdsOpen("output/LinkFilt010321.gds")

# LD pruning - Try different LD thresholds for sensitivity analysis
set.seed(32) # Hopefully this ensure the filtering produces the exact same SNPs each time it's redone
gds_LD <- snpgdsLDpruning(gds, method="corr", ld.threshold = 0.5, slide.max.bp = 500000, autosome.only = FALSE)

# How many SNPs have we filtered down to?
LDFilt <- unlist(gds_LD)
length(LDFilt)


## Subset main data frame using independent SNPs - better for relatedness and PCA
dasgen <- subset(dasgen, rownames(dasgen) %in% LDFilt)
dasgen <- as.data.frame(t(dasgen))

# Check samples
nsamps <- nrow(dasgen)
nsamps

# Check snps
nsnps <- ncol(dasgen)
nsnps




## Get population names
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
popns <- samps$popn
table(samps$popn)



# Close the stupid gds file
snpgdsClose(gds)
detach("package:SNPRelate", unload=TRUE)



########################################################################################################


# Output SNP Locus names for future reference
write.csv(colnames(dasgen),paste("output/",length(LDFilt),"_snp_IDs_",currentDate,".csv", sep=""),row.names = FALSE)






################################
# Check genotyping error rates #
################################

# Extract all tech reps
TRs <- rownames(dasgen)[grep(pattern="x",x=rownames(dasgen))]
TRmates <- c("DHAL0023", "DHAL0068", "DHAL0169","DHAL0183", "DHAL0240","DHAL0249","DHAL0253", "DHAL0349",
             "DHAL0358", "DHAL0377","DHAL0507", "DHAL0510", "DHAL0522","DHAL0645","DHAL0755")
TRs <- c(TRs,TRmates)
dasgenTRs <- subset(dasgen, (rownames(dasgen) %in% TRs))

# Set up data frame to dump results into
pct_similr <- c(rep(NA,15))

# Make list of pairs of all TRs
selectd <- list(c(1:2),c(3:4),c(5:6),c(7:8),c(9:10),c(11:12),c(13:14),c(15:16),c(17:18),c(19:20),c(21:22),c(23:24),
                c(25:26),c(27:28),c(29:30))

# Run loop that subsets data to selected pair of TRs and calculates number of differences
for(i in selectd){
    pair <- dasgenTRs[i,]
    pair <- pair[,!sapply(pair,function(x) any(is.na(x)))] # Remove snps with NA vals
    pairdiff <- (pair[1,])-(pair[2,])
    pairdiff <- abs(pairdiff)
    pct_similr[i] <- ((2*length(pairdiff))-sum(pairdiff))/(2*length(pairdiff))*100
}

diffs <- pct_similr[c(TRUE, FALSE)]
mean(pct_similr) # mean is 99.99% similarity between TRs
library("plotrix") # Package with standard error function
std.error(pct_similr) # standard error is 0.003%




####################################################################
# Create hierarchical clustering dendrogram to visualise structure #
####################################################################

# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes


## Do population level dendrogram
library(ape)
# Convert to genpop object
genpopn <- genind2genpop(genotypes, quiet = FALSE, process.other = TRUE,
                         other.action = mean)

# Plot population genetic distances as bootstrapped dendrogram 
library(dplyr)
library(poppr)
set.seed(33)
genotypes %>%
  genind2genpop %>%
  aboot(cutoff = 50, quiet = TRUE, sample = 1000, root = NULL, distance = provesti.dist) #looks good

# Get population genetic distance (1=nei, 4=euclidean)
popgens <- dist.genpop(genpopn, method=1)
hcpop <- hclust(popgens)
plot(as.phylo(hcpop),cex=1,show.tip.label=TRUE)

## Do sample level dendrogram
library(poppr)
bitdist <- bitwise.dist(genotypes, percent = TRUE, mat = FALSE)
hc <- hclust(bitdist)
plot(hc, labels=NULL)
# Tech Reps and Biol Reps are paired up very strongly in this plot. So not likely to be much influence
# from different runs/lanes/plates.
plot(hc, labels=genotypes@pop) # Same but with pops as names

# Plot as a phylogeny
plot(as.phylo(hc),cex=1,show.tip.label=TRUE)


# Output population dendrogram for manuscript supplementary material
pdffilename <- paste("output/dasy_dendro_pop", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=5, height=7)
plot(as.phylo(hcpop),cex=1,show.tip.label=TRUE)
dev.off()

# Output sample dendrogram for manuscript supplementary material
pdffilename <- paste("output/dasy_dendro_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=15, height=65)
plot(as.phylo(hc),cex=1,show.tip.label=TRUE)
dev.off()

# Plot as classic pop dendrogram
pdffilename <- paste("output/dasy_dendro_pop_2_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=65, height=15)
plot(hc, labels=genotypes@pop) # with pops as names
dev.off()



## Check allele frequencies
af <- minorAllele(genotypes)
length(af[af>0.05])





###############################
# Remove technical replicates #
###############################
rownames(dasgen)[grep(pattern="x",x=rownames(dasgen))]
dasgen = subset(dasgen, !(rownames(dasgen) %in% rownames(dasgen)[grep(pattern="x",x=rownames(dasgen))]))


## Get population names
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
popns <- samps$popn
table(samps$popn)




# Output whole SNP-sample matrix to use for future analyses
# write.csv(dasgen,"output/dasy_genos_010321.csv",row.names=TRUE)

# This spreadsheet contains loci under selection
# Can possibly find them using an outlier test for general SNPs under selection
# and then use the LFMM2 function to find SNPs specifically associated with cane toads




#############################################################
#       Create LEA snmf project and search for associations #
#############################################################

### Prepare imputed dataset for use in outlier test and lfmm
# Requires using higher frequency SNPs, and for lfmm no missing data

# Here I'm calling Y the genotype df, and X the environmental df
# (Although I also refer to x as some number of iterations/runs/k-values...)
Ystart <- dasgen

# Need to remove SNPs with low minor allele frequencies from the analysis
Ystart <- as.data.frame(t(Ystart))
Ystart$missingperloc <- rowSums(is.na(Ystart))
Ystart$samplestyped <- ncol(Ystart[1:(ncol(Ystart)-1)])-Ystart$missingperloc
Ystart$callrate <- Ystart$samplestyped/(Ystart$samplestyped+Ystart$missingperloc)
Ystart$allelecount <- rowSums(Ystart[1:(ncol(Ystart)-2)],na.rm=TRUE)
Ystart$alf <- Ystart$allelecount/(2*Ystart$samplestyped)
Ystart$alleleA <- 1-Ystart$alf
Ystart$alf <- ifelse(Ystart$alf>0.5, Ystart$alleleA, Ystart$alf)
Ystart$alleleA <- 1-Ystart$alf
Ystart <- Ystart[Ystart$alf>=0.025,] # Remove uncommon SNPs

# Remove extraneous columns
head(Ystart[,(ncol(Ystart)-9):(ncol(Ystart))]) # shows head of last x columns
Ystart <- Ystart[,1:(ncol(Ystart)-6)] # Removes last x columns
head(Ystart[,(ncol(Ystart)-9):(ncol(Ystart))]) # All good


# BiocManager::install("LEA")
library(LEA)

## Ok so I've just got Ystart as standard 0,1,2 genotype format (see rows vs columns stuff below - geno file should have snps in rows)
# Ystart[Ystart=="2,2"] <- 2 # Homozygote minor/alternate
# Ystart[Ystart=="1,2"] <- 1 # Heterozygote
# Ystart[Ystart=="1,1"] <- 0 # Homozygote major/reference

# Format for LEA package and check cross-entropy
Ystart[1:10,1:10] # Need to change NA values to 9
Ystart[is.na(Ystart)] <- 9
Ystart[1:10,1:10]
# Ystart <- as.data.frame(t(Ystart)) # One row for each SNP, as .geno file should be!!

# Create stupid geno file for imputation to allow fitting of the lfmm
write.geno(as.data.frame(t(Ystart)), "output/genosQuolls030322.geno") # IF USING WRITE.GENO, Ystart NEEDS TO BE SNPS IN COLUMNS (FFS)
# write.table(Ystart, "genotypesVCH300620.geno", row.names=FALSE, col.names=FALSE, sep="") # Alt method. Here snps are in rows

# Estimate individual ancestry coefficients and ancestral allele frequencies
# Makes a directory/folder on the hard drive with all the run info in it
# Runs (latent factors = ancestral populations[?]) K = 1:x with cross-entropy calcs, each done with i
# repetitions.
project1 = NULL
project1 = snmf("output/genosQuolls030322.geno", K=1:15, entropy = TRUE, repetitions = 10,
                project = "new", ploidy=2) #, iterations=100000, tolerance = 0.00000000001)
project1 = load.snmfProject("output/genosQuolls030322.snmfProject")

# plot cross-entropy criterion of all runs of the project
plot(project1, lwd = 5, col = "red", pch=1)

# get the cross-entropy of each run for K = x
ce = cross.entropy(project1, K = 10) # CHOOSE K YOU WANT

# select the run with the lowest cross-entropy
best = which.min(ce)

# take the ancestry coefficients for the best run where K = x
qmatrix2 = Q(project1, K = 10, run = best)

# Admixture barplot
barplot(t(qmatrix2), col = rainbow(3), border = NA, space = 0,
        xlab = "Individuals", ylab = "Admixture coefficients")


### Get FST values and run outlier test (as per Martins et al. 2016)

# Define fst function to calculate differentiation based on the ancestral allele frequencies
fst = function(project,run = 1, K, ploidy = 2){
  library(LEA)
  l = dim(G(project, K = K, run = run))[1]
  q = apply(Q(project, K = K, run = run), MARGIN = 2, mean)
  if (ploidy == 2) {
    G1.t = G(project, K = K, run = run)[seq(2,l,by = 3),]
    G2.t = G(project, K = K, run = run)[seq(3,l,by = 3),]
    freq = G1.t/2 + G2.t}
  else {
    freq = G(project, K = K, run = run)[seq(2,l,by = 2),]}
  H.s = apply(freq*(1-freq), MARGIN = 1, FUN = function(x) sum(q*x))
  P.t = apply(freq, MARGIN = 1, FUN = function(x) sum(q*x))
  H.t = P.t*(1-P.t)
  return(1-H.s/H.t)
}

# Compute fst values
fst.values = fst(project1, K=10, run=best)

#convert fst values into z-scores (absolute values) 
n = dim(Q(project1, K = 10, run = best))[1]
fst.values[fst.values<0] = 0.000001
K = 10
z.scores = sqrt(fst.values*(n-K)/(1-fst.values))

#Compute the GIF
K=10
lambda = median(z.scores^2)/qchisq(1/2, df = K-1)
lambda

# compute adjusted p-values from the combined z-scores
adj.p.values = pchisq(z.scores^2/25, df = K-1, lower = FALSE)

#histogram of p-values
hist(adj.p.values, col = "red")

## FDR control: Benjamini-Hochberg at level q
L = ncol(Ystart)
q = 0.001
w = which(sort(adj.p.values) < q * (1:L)/L)
candidates = order(adj.p.values)[w]

# Plot
ps <- as.data.frame(-log10(adj.p.values))
colnames(ps) <- "pvalues"
ps$pvalues[!is.finite(ps$pvalues)] <- NA

library(ggplot2)
manplot <- ggplot() +
  geom_point(data=ps, aes(y=pvalues, x=as.numeric(rownames(ps))), colour="grey75") +
  geom_point(data=as.data.frame(ps[c(candidates),]),
             aes(y=ps$pvalues[candidates],
                 x=as.numeric(rownames(ps)[candidates])), colour="red") +
  theme_bw() +
  xlab("Locus") + 
  ylab("-log10(p-values)") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5))

manplot

# Output manhattan plot to PDF
pdffilename <- paste("output/manhattan_plot_outliers_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=9, height=5)
manplot
dev.off()

## Keep list of outlier SNPs to remove later
outlierSNPs <- rownames(Ystart)[candidates]





########################
# Run LFMM against toads

## Impute missing data using ancestry information
impute(project1, "output/genosQuolls030322.geno", method="mode", K=10, run=best)

# Read imputed file back in
Y <- as.data.frame(read.lfmm("output/genosQuolls030322.lfmm_imputed.lfmm")) # Check filename in directory
dim(Y)
# Y <- as.data.frame(t(Y))


## Organise X (environmental) variables
library(lubridate)
tst <- read.csv("output/sampinfo250221_toadtimes.csv")
tst <- tst[tst$sample %in% rownames(dasgen),]

# # Retain only samples in NT and QLD, and remove island populations
# tst <- tst[tst$state!="WA",]
# tst <- tst[!grepl("Island", tst$popn),]
# tst <- tst[!grepl("Groote", tst$popn),]

# Cut genotypes dataset to same samples as timesincetoad
rownames(Y) <- rownames(dasgen)
Y[1:10,1:10]
Y <- Y[rownames(Y) %in% tst$sample,]
identical(rownames(Y),tst$sample)

# Define dataset for association with genotypes
X <- tst[,c(1,9)]
X$sample==rownames(Y) # make sure order of samples in the environment file is the same as the geno file
rownames(X) <- X$sample
X$sample <- NULL

# Remove the stupid project
#remove.snmfProject("output/genotypesVCH.snmfProject")

# Create env formatted file
write.table(X, file="output/toads_treatment030322.env", row.names=FALSE, col.names=FALSE)

# This function uses the newer exact least-squares approach rather than the interminable MCMC estimation algorithm in original lfmm function
obj.lfmm = lfmm2(input=Y, env=X, K = 10)

# Save lfmm object
saveRDS(obj.lfmm, "lfmm_object_quolls_toads_030322")
obj.lfmm <- readRDS("lfmm_object_quolls_toads_030322")

# Run association tests (linear model) between snps and environment
pv <- lfmm2.test(object=obj.lfmm, input=Y, env=X, genomic.control=TRUE, linear=TRUE)
# genomic.control is by robust estimate of the genomic inflation factor

# Extract pvalues for each environmental variable
L = length(pv$pvalues)
fdr.level = 1e-3 # This is the false discovery rate (eg 1e-2 = 0.01 = 1 SNP in 100 falsely discovered)
w = which(sort(pv$pvalues) < fdr.level*(1:L)/L)
candidates = order(pv$pvalues)[w]
length(candidates)

# Retain candidate toad-associated SNPs
toadSNPs <- rownames(Ystart)[candidates]

# Plot
ps2 <- as.data.frame(-log10(pv$pvalues))
colnames(ps2) <- "pvalues"
ps2$pvalues[!is.finite(ps2$pvalues)] <- NA
rownames(ps2) <- c(1:nrow(ps2))

library(ggplot2)
manplot2 <- ggplot() +
  geom_point(data=ps2, aes(y=pvalues, x=as.numeric(rownames(ps2))), colour="grey75") +
  geom_point(data=as.data.frame(ps2[c(candidates),]),
             aes(y=ps2$pvalues[candidates],
                 x=as.numeric(rownames(ps2)[candidates])), colour="red") +
  theme_bw() +
  xlab("Locus") + 
  ylab("-log10(p-values)") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  scale_y_continuous(limits=c(0,240))

manplot2


# Output manhattan plot to PDF
pdffilename <- paste("output/manhattan_plot_cane_toad_lfmm_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=9, height=5)
manplot2
dev.off()


# End lfmm




################################
## Combine LFMM and outlier SNPs

## Check overlap between toadSNPs and outlier SNPs
length(c(outlierSNPs,toadSNPs))
length(unique(c(outlierSNPs,toadSNPs)))
length(intersect(outlierSNPs,toadSNPs))

## Combine into a dataframe
library(dplyr)
outlierAndToadSNPs <- full_join(data.frame(key=toadSNPs, toadSNPs, stringsAsFactors = FALSE),
                                data.frame(key=outlierSNPs, outlierSNPs, stringsAsFactors = FALSE),
                                by="key") %>% select(-key)

# Output all the cand snps
write.csv(outlierAndToadSNPs,"output/all_cand_SNPs_030322.csv", row.names = FALSE)
write.csv(outlierSNPs,"output/outlier_cand_SNPs_030322.csv", row.names = FALSE)
write.csv(toadSNPs,"output/toad_cand_SNPs_030322.csv", row.names = FALSE)


### Remove all SNPs showing signals of selection from genotypes dataset
toremove <- unique(c(outlierSNPs,toadSNPs))
dasgen <- as.data.frame(t(dasgen))
dasgen <- subset(dasgen[!rownames(dasgen) %in% toremove , ])
dasgen <- as.data.frame(t(dasgen))


# Get values for manuscript
candy1 <- read.csv("output/outlier_cand_SNPs_030322.csv",header = TRUE) # 259 SNPs identified by outlier analysis
candy2 <- read.csv("output/toad_cand_SNPs_030322.csv",header = TRUE) # 363 SNPs identified by LFMM analysis
length(unique(c(candy1$x,candy2$x))) # 485 unique SNPs identified in total
length(c(candy1$x,candy2$x)[duplicated(c(candy1$x,candy2$x))]) # 137 SNPs identified by both methods

candy <- read.csv("output/all_cand_SNPs_030322.csv",header = TRUE)
sort(table(candy$Scaffold))
length(unique(candy$Scaffold))

# # Output whole SNP-sample matrix to use for relatedness estimation
# write.csv(dasgen,"output/dasy_genos_030322.csv",row.names=TRUE)






###########################################
# Remove samples that are closely related # (not used in quoll manuscsript)
###########################################

# # Coancestry software package method done outside R 
# coan <- genind2df(genotypes, pop=NULL, sep="", oneColPerAll = TRUE)
# coan[1:10,1:10]
# popn <- coan$pop
# coan$pop <- NULL
# coan[] <- lapply(coan, function(x) {
#   if(is.character(x)) as.numeric(x) else x
# })
# coan[is.na(coan)] <- 0
# coan[1:10,1:10]
# popn <- abbreviate(popns, minlength = 2, use.classes = TRUE,
#                    dot = FALSE, strict = TRUE,
#                    method = c("left.kept"), named = TRUE)
# popsamp <- paste(popn,"_",rownames(coan), sep="")
# coan <- cbind(popsamp,coan)
# write.table(coan, "output/coancestry_genos_030322.txt", row.names=FALSE, col.names=TRUE, sep="\t", quote=FALSE)



##############################################################
### Relatedness using Weir & Goudet 2017 / Goudet et al. 2018 methods
library(hierfstat)

# Clear environment
rm(list = ls())
currentDate <- Sys.Date() # Add current date to environment for output of file names with date


# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_030322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn

# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Make population groups
length(unique(genotypes@pop))
sort(table(genotypes@pop))
genosEA2017 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[22],]@tab)
genosMareeba <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[23],]@tab)
genosMillChist <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[24],]@tab)
genosMarBar <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[25],]@tab)
genosBach <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[26],]@tab)
genosKoolan <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[27],]@tab)
genosDar2000 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[28],]@tab)
genosKingLE <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[29],]@tab)
genosWeip <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[30],]@tab)
genosKingLW <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[31],]@tab)
genosAI16 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[32],]@tab)
genosEA2003 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[33],]@tab)
genosMornin <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[34],]@tab)
genosPobass <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[35],]@tab)
genosGrooteW <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[36],]@tab)
genosAI05 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[37],]@tab)
genosArtR <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[38],]@tab)
genosKapa <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[39],]@tab)
genosMarch <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[40],]@tab)
genosOutDar <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[41],]@tab)

poops <- list(genosEA2017,genosMareeba,genosMillChist,genosMarBar,genosBach,genosKoolan,genosDar2000,
              genosKingLE,genosWeip,genosKingLW,genosAI16,genosEA2003,genosMornin,genosPobass,
              genosGrooteW,genosAI05,genosArtR,genosKapa,genosMarch,genosOutDar)

# Run loop using the Goudet estimator - just runs off the subsetted dasgen df (instead of stupid gds format)
relatedness <- as.data.frame(matrix(nrow=3,ncol=3))
colnames(relatedness) <- c("ind1","ind2","relatedness")

library(otuSummary)
for(i in poops){
  rels <- subset(dasgen,rownames(dasgen) %in% i)
  rels <- beta.dosage(rels,inb=FALSE,Mb=FALSE)
  rels <- matrixConvert(rels, colname = c("ind1", "ind2", "relatedness"))
  relatedness <- rbind(relatedness,rels)
}

# Finalise and output df
relatedness <- relatedness[4:nrow(relatedness),]
write.csv(relatedness,"output/relatedness030322.csv",row.names=FALSE)

# Find pairs above some threshold of relatedness
relateds <- relatedness[relatedness$relatedness>0.25,]
relateds <- relateds[4:nrow(relateds),]
library(dplyr)
samppop <- samps[ , c(1,4)]
relateds$sample <- relateds$ind1
relateds <- left_join(relateds,samppop,by="sample")
table(relateds$popn)

write.csv(relateds,"output/highly_related_individuals_030322.csv",row.names=FALSE)

## Remove one of each pair from the overall dataset
sampstoremove <- unique(relateds$ind1)

dasgen <- subset(dasgen[!rownames(dasgen) %in% sampstoremove , ])




############################################################
# Output whole SNP-sample matrix to use FOR REST OF ANALYSES
write.csv(dasgen,"output/dasy_genos_040322.csv",row.names=TRUE)








########################################################################################

#################################################################

###########################################

###################################
# Start doing actually useful stuff
###################################

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn

geno.types <- dasgen

# Check some popn size stuff
poptab <- as.data.frame(table(samps$popn))
nrow(poptab[poptab$Freq>=8,]) # 23 pops with >=6 samples; 20 with >=8 samples
nrow(poptab[poptab$Freq<4,])
length(samps$treatment[samps$treatment=="post-toad"]) # 51 toad-exposed individuals to play with






#########################################################
# Make MDS plot of genetic distance between populations #
#########################################################

# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Check missingness
library(poppr)
div <- summary(genotypes) # 5 mins with large datasets
div$NA.perc # 1.16% missing data - good
hist(rowSums(is.na(geno.types)))


# # Convert to genpop object
# dasgenpop <- genind2genpop(genotypes, quiet = FALSE, process.other = TRUE,
#                            other.action = mean)
# # Get population genetic distance (1=nei, 4=euclidean)
# PopnGenDist <- dist.genpop(dasgenpop, method=1, diag=TRUE, upper=FALSE)

# Individual genetic distance
# PopnGenDist <- dist(wagenotypes, method = "euclidean", diag = FALSE, upper = FALSE, p = 2)
PopnGenDist <- prevosti.dist(genotypes)
PopnGenDist <- as.matrix(PopnGenDist)

## Classical (metric) MDS plot using population distance matrix
PopnGenDist <- as.matrix(PopnGenDist)
fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2) # classical MDS of distance plot; k=number of dimensions
popeig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
popeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]

# Use MDS output (fit) to plot principal coordinates using GGPLOT2
fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot

# # Get state IDs for colours (popn level mds)
# states <- samps[match(unique(samps$popn), samps$popn),]
# fitt$popn <- rownames(fitt)
# fitt <- merge(fitt, states, by="popn")
# rownames(fitt) <- fitt$popn
# fitt$sample <- fitt$lat <- fitt$long <- fitt$sex <- fitt$region <- fitt$date <- fitt$treatment <- NULL

fitt$state <- samps$state

library(ggrepel) # Can use this to plot with pretty labels that don't touch (repel) each other
library(ggplot2)
set.seed(32)
mdspopgendist <- ggplot(fitt, aes(V1,V2)) +
  geom_vline(xintercept=0, size=0.3, color="darkgrey") +
  geom_hline(yintercept=0, size=0.3, color="darkgrey") +
  geom_point(aes(fill=state), colour="black", shape=21, size=5) +
  scale_fill_manual(values=c("#56B4E9","#009E73","#E69F00")) +
  # geom_text_repel(aes(label=rownames(fitt)), force=5, box.padding = unit(0.8, 'lines'), max.overlaps = 30, max.iter = 5000, point.padding=0.2, segment.color = "grey60") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  xlab(paste("Coordinate 1 ","(",popeig1,"%)",sep="")) + 
  ylab(paste("Coordinate 2 ","(",popeig2,"%)",sep="")) +
  guides(fill="none") +
  annotate(geom="text", x=-0.005,y=0.0,label="Northern Territory", size=5) +
  annotate(geom="text", x=0.0055,y=0.0225,label="Queensland", size=5) +
  annotate(geom="text", x=0.017,y=-0.0045,label="Western Australia", size=5)
  # scale_x_continuous(limits=c(-0.03,0.04)) +
  # scale_y_continuous(limits=c(-0.05,0.015))
mdspopgendist


# Output MDS to PDF
pdffilename <- paste("output/MDS_pop_gen_distance", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=6.5, height=5)
mdspopgendist
dev.off()



### Do MDS plot doing each state separately

## WA
wapops <- unique(samps$popn[!samps$state=="WA"])
wagenotypes <- genotypes
for(i in wapops){
  wagenotypes <- wagenotypes[wagenotypes@pop!=i,]
}
# dasgenpop <- genind2genpop(wagenotypes, quiet = FALSE, process.other = TRUE,
#                            other.action = mean)
# PopnGenDist <- dist.genpop(dasgenpop, method=1, diag=TRUE, upper=FALSE)
# PopnGenDist <- as.matrix(PopnGenDist)
# fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2) # classical MDS of distance plot; k=number of dimensions
# waeig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
# wapeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]
# fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot

# Individual genetic distance
PopnGenDist <- prevosti.dist(wagenotypes)
PopnGenDist <- as.matrix(PopnGenDist)
fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2, add=TRUE) # classical MDS of distance plot; k=number of dimensions
round(fitt$eig*100/sum(fitt$eig),1) # Percentage of variance explained by each axis
waeig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
wapeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]
fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot
fitt$region <- samps[samps$state=="WA",]$region
fitt$region[c(85:93)] <- "Koolan Island"

library(ggrepel) # Can use this to plot with pretty labels that don't touch (repel) each other
library(ggplot2)
set.seed(32)
mdspopgendist <- ggplot(fitt, aes(V1,V2)) +
  geom_vline(xintercept=0, size=0.3, color="darkgrey") +
  geom_hline(yintercept=0, size=0.3, color="darkgrey") +
  geom_point(aes(fill=region), colour="black", shape=21, size=5) +
  scale_fill_manual(values=c("#E69F00","#E69F00","#E69F00")) +
  # geom_text_repel(aes(label=rownames(fitt)), colour="grey35",force=9, box.padding = unit(0.8, 'lines'),
  #                 max.overlaps = 30, max.iter = 4000, point.padding=0.1, segment.color = "grey60") +
  guides(fill="none") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  xlab(paste("Coordinate 1 ","(",waeig1,"%)",sep="")) + 
  ylab(paste("Coordinate 2 ","(",wapeig2,"%)",sep="")) +
  annotate(geom="text", x=-0.0025,y=0.002,label="Kimberley", size=5) +
  annotate(geom="text", x=-0.0015,y=-0.0175,label="Mornington Sanctuary", size=5) +
  annotate(geom="text", x=-0.003,y=0.035,label="Koolan Island", size=5) +
  annotate(geom="text", x=-0.03,y=0.0065,label="Pilbara", size=5) +
  scale_x_continuous(breaks=c(-0.03,-0.01,0.01))

mdspopgendist


# Output MDS to PDF
pdffilename <- paste("output/MDS_WA_gen_distance", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=6.5, height=5)
mdspopgendist
dev.off()



## NT
ntpops <- unique(samps$popn[!samps$state=="NT"])
ntgenotypes <- genotypes
for(i in ntpops){
  ntgenotypes <- ntgenotypes[ntgenotypes@pop!=i,]
}

# dasgenpop <- genind2genpop(ntgenotypes, quiet = FALSE, process.other = TRUE,
#                            other.action = mean)
# PopnGenDist <- dist.genpop(dasgenpop, method=1, diag=TRUE, upper=FALSE)
# PopnGenDist <- as.matrix(PopnGenDist)
# fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2) # classical MDS of distance plot; k=number of dimensions
# nteig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
# ntpeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]
# fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot

# Individual genetic distance
PopnGenDist <- prevosti.dist(ntgenotypes)
PopnGenDist <- as.matrix(PopnGenDist)
fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2, add=TRUE) # classical MDS of distance plot; k=number of dimensions
round(fitt$eig*100/sum(fitt$eig),1) # Percentage of variance explained by each axis
nteig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
ntpeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]
fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot
fitt$region <- samps[samps$state=="NT",]$region

library(ggrepel) # Can use this to plot with pretty labels that don't touch (repel) each other
library(ggplot2)
set.seed(32)
mdspopgendist <- ggplot(fitt, aes(V1,V2)) +
  geom_vline(xintercept=0, size=0.3, color="darkgrey") +
  geom_hline(yintercept=0, size=0.3, color="darkgrey") +
  geom_point(aes(fill=region), colour="black", shape=21, size=5) +
  scale_fill_manual(values=c("#56B4E9","#56B4E9","#56B4E9","#56B4E9","#56B4E9")) +
  # geom_text_repel(aes(label=rownames(fitt)), colour="grey35",force=9, box.padding = unit(0.8, 'lines'),
  #                 max.overlaps = 30, max.iter = 4000, point.padding=0.1, segment.color = "grey60") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  xlab(paste("Coordinate 1 ","(",nteig1,"%)",sep="")) + 
  ylab(paste("Coordinate 2 ","(",ntpeig2,"%)",sep="")) +
  annotate(geom="text", x=-0.0155,y=0.0175,label="Groote \nEylandt", size=5) +
  annotate(geom="text", x=0.006,y=0.01,label="Northern Territory \nmainland", size=5) +
  annotate(geom="text", x=-0.013,y=-0.005,label="Pellew \nIslands", size=5) +
  annotate(geom="text", x=-0.018,y=-0.02,label="Marchinbar Island", size=5) +
  scale_y_continuous(breaks=c(-0.02,0.00,0.02)) +
  guides(fill="none")
mdspopgendist


# Output MDS to PDF
pdffilename <- paste("output/MDS_NT_gen_distance", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=6.5, height=5)
mdspopgendist
dev.off()





## QLD
qldpops <- unique(samps$popn[!samps$state=="QLD"])
qldgenotypes <- genotypes
for(i in qldpops){
  qldgenotypes <- qldgenotypes[qldgenotypes@pop!=i,]
}

# dasgenpop <- genind2genpop(qldgenotypes, quiet = FALSE, process.other = TRUE,
#                            other.action = mean)
# PopnGenDist <- dist.genpop(dasgenpop, method=1, diag=TRUE, upper=FALSE)
# PopnGenDist <- as.matrix(PopnGenDist)
# fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2) # classical MDS of distance plot; k=number of dimensions
# qldeig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
# qldpeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]
# fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot

# Individual genetic distance
PopnGenDist <- prevosti.dist(qldgenotypes)
PopnGenDist <- as.matrix(PopnGenDist)
fitt <- cmdscale(PopnGenDist, eig=TRUE, k=2, add=TRUE) # classical MDS of distance plot; k=number of dimensions
round(fitt$eig*100/sum(fitt$eig),1) # Percentage of variance explained by each axis
qldeig1 <- round(fitt$eig*100/sum(fitt$eig),1)[1] # Percentage of variance explained by first and second axes
qldpeig2 <- round(fitt$eig*100/sum(fitt$eig),1)[2]
fitt <- as.data.frame(fitt$points) # Needs to be a stupid data.frame not a matrix for ggplot
fitt$region <- samps[samps$state=="QLD",]$popn


library(ggrepel) # Can use this to plot with pretty labels that don't touch (repel) each other
library(ggplot2)
set.seed(32)
mdspopgendist <- ggplot(fitt, aes(V1,V2)) +
  geom_vline(xintercept=0, size=0.3, color="darkgrey") +
  geom_hline(yintercept=0, size=0.3, color="darkgrey") +
  geom_point(aes(fill=region), colour="black", shape=21, size=5) +
  scale_fill_manual(values=c("#009E73","#009E73","#009E73","#009E73")) +
  # geom_text_repel(aes(label=rownames(fitt)), colour="grey35",force=9, box.padding = unit(0.8, 'lines'),
  #                 max.overlaps = 30, max.iter = 4000, point.padding=0.1, segment.color = "grey60") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  xlab(paste("Coordinate 1 ","(",qldeig1,"%)",sep="")) + 
  ylab(paste("Coordinate 2 ","(",qldpeig2,"%)",sep="")) +
  annotate(geom="text", x=-0.0025,y=-0.0055,label="Black Mountain \n/ Hope Vale", size=5) +
  annotate(geom="text", x=-0.009,y=0.003,label="Mareeba", size=5) +
  annotate(geom="text", x=0.001,y=0.007,label="Weipa", size=5) +
  scale_y_continuous(limits=c(-0.0082,0.0082),breaks=c(-0.008,0.00,0.008)) +
  guides(fill="none")

mdspopgendist


# Output MDS to PDF
pdffilename <- paste("output/MDS_QLD_gen_distance", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=6.5, height=5)
mdspopgendist
dev.off()







#######################################################
# Get standard genetic parameters for all populations #
#######################################################

# Remove the small (< 6 individuals) populations first
smallpops <- as.data.frame(table(popns))[as.data.frame(table(popns))$Freq<6,]
smallpops <- as.character(smallpops$popns)
for(i in smallpops){
  genotypes <- genotypes[genotypes@pop!=i,]
}

## Separate out your treatments for calculating pop gen parameters individually
toadex <- samps[samps$treatment=="post-toad",]
toadnaive <- samps[samps$treatment=="pre-toad",]
islands <- samps[samps$treatment=="island",]
toadex <- genotypes[row.names(genotypes@tab) %in% toadex$sample]
toadnaive <- genotypes[row.names(genotypes@tab) %in% toadnaive$sample]
islands <- genotypes[row.names(genotypes@tab) %in% islands$sample]
table(toadex@pop)
table(toadnaive@pop)
table(islands@pop)

## Gstudio package
# require(devtools)
# install_github("dyerlab/popgraph")
library(popgraph)
# install_github("dyerlab/gstudio")
library(gstudio)

## Calcs for toad-naive populations
# Convert data to gstudio format
data4gstudio <- genind2df(toadnaive, sep="", usepop=TRUE, oneColPerAll=FALSE)
data4gstudio[data4gstudio==22] <- 2
data4gstudio[data4gstudio==12] <- 1
data4gstudio[data4gstudio==11] <- 0
data4gstudio[1:10,1:10]

# Read into gstudio
write.csv(data4gstudio, "output/gstuddat.csv")
gstudred <- read_population("output/gstuddat.csv", type = "snp", locus.columns = 3:(ncol(data4gstudio)+1))
gstudred[1:10,1:6]

# Group by population
clades <- partition(gstudred, stratum = "pop")
names(clades)
lapply(clades, dim) # looking good

# Get genetic stats by locus for each population
gstua <- genetic_diversity(gstudred, stratum="pop", mode = "A", small.N = TRUE) # Will take ~6 mins
gstuae <- genetic_diversity(gstudred, stratum="pop", mode = "Ae", small.N = TRUE) # Will take ~6 mins
gstuhe <- genetic_diversity(gstudred, stratum="pop", mode = "He", small.N = TRUE) # Will take ~7 mins
gstuho <- genetic_diversity(gstudred, stratum="pop", mode = "Ho", small.N = TRUE) # Will take ~7 mins
gstufis <- genetic_diversity(gstudred, stratum="pop", mode = "Fis", small.N = TRUE) # Will take ~7 mins
gstufpe <- genetic_diversity(gstudred, stratum="pop", mode = "Pe", small.N = TRUE) # Will take ~7 mins
gstuhe[1:30,1:3]
identical(gstuhe[['Locus']],gstufis[['Locus']]) # Good

# Bind all results together
allstats <- cbind(gstua,gstuae$Ae,gstuhe$He,gstuho$Ho,gstufis$Fis,gstufpe$Pe)
allstats[1:10,]

# Change all infinite values to NA
any(is.infinite(allstats$`gstuae$Fis`)) # There are infinite values messing up the mean calcs
library(data.table)
for (j in 1:ncol(allstats)) set(allstats, which(is.infinite(allstats[[j]])), j, NA)

# Obtain means for each population
allstatspop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=mean, na.rm=TRUE, na.action=NULL)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe")
allstatspop[1:nrow(allstatspop),]
round(allstatspop[,2:7],3)

# Calculate number of SNPs used for deriving stats in each population
count_nas <- function(arg1) {
  return(sum(is.na(arg1)))
}
aggregate(allstats[,3:8],by=list(allstats$Stratum),FUN=count_nas)

# Obtain standard deviation for each popn
library("plotrix")
allstatsSDpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=sd, na.rm=TRUE)
colnames(allstatsSDpop) <- c("Pop","ASD","AeSD","HeSD","HoSD","FisSD","PeSD")

 # Obtain standard errors for each popn
library("plotrix")
allstatsSEpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=std.error, na.rm=TRUE)
colnames(allstatsSEpop) <- c("Pop","ASE","AeSE","HeSE","HoSE","FisSE","PeSE")
round(allstatsSEpop[,2:7],3)

# Cbind SDs and SEs to Means
allstatspop <- cbind(allstatspop,
                     allstatsSEpop$ASE, allstatsSEpop$AeSE,allstatsSEpop$HeSE,allstatsSEpop$HoSE,allstatsSEpop$FisSE,allstatsSEpop$PeSE,
                     allstatsSDpop$ASD, allstatsSDpop$AeSD,allstatsSDpop$HeSD,allstatsSDpop$HoSD,allstatsSDpop$FisSD,allstatsSDpop$PeSD)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe",
                           "ASE","AeSE","HeSE","HoSE","FisSE","PeSE",
                           "ASD","AeSD","HeSD","HoSD","FisSD","PeSD")
toadnaivestats <- allstatspop
toadnaivestats <- cbind(toadnaivestats$Pop,round(toadnaivestats[,2:19],3))
colnames(toadnaivestats)[1] <- "Locality"

# Output to csv file
filename <- paste("output/popgenstatsNAIVE",currentDate,".csv",sep="")
write.csv(toadnaivestats,filename)


## Calcs for toad-exposed populations
# Convert data to gstudio format
data4gstudio <- genind2df(toadex, sep="", usepop=TRUE, oneColPerAll=FALSE)
data4gstudio[data4gstudio==22] <- 2
data4gstudio[data4gstudio==12] <- 1
data4gstudio[data4gstudio==11] <- 0
data4gstudio[1:10,1:10]

# Read into gstudio
write.csv(data4gstudio, "output/gstuddat.csv")
gstudred <- read_population("output/gstuddat.csv", type = "snp", locus.columns = 3:(ncol(data4gstudio)+1))
gstudred[1:10,1:6]

# Group by population
clades <- partition(gstudred, stratum = "pop")
names(clades)
lapply(clades, dim) # looking good

# Get genetic stats by locus for each population
gstua <- genetic_diversity(gstudred, stratum="pop", mode = "A", small.N = TRUE) # This will take ~7 mins
gstuae <- genetic_diversity(gstudred, stratum="pop", mode = "Ae", small.N = TRUE) # This will take ~7 mins
gstuhe <- genetic_diversity(gstudred, stratum="pop", mode = "He", small.N = TRUE) # This will take ~7 mins
gstuho <- genetic_diversity(gstudred, stratum="pop", mode = "Ho", small.N = TRUE) # This will take ~7 mins
gstufis <- genetic_diversity(gstudred, stratum="pop", mode = "Fis", small.N = TRUE) # This will take ~7 mins
gstufpe <- genetic_diversity(gstudred, stratum="pop", mode = "Pe", small.N = TRUE) # This will take ~7 mins
gstuhe[1:30,1:3]
identical(gstuhe[['Locus']],gstufis[['Locus']]) # Good

# Bind all results together
allstats <- cbind(gstua,gstuae$Ae,gstuhe$He,gstuho$Ho,gstufis$Fis,gstufpe$Pe)
allstats[1:10,]

# Change all infinite values to NA
any(is.infinite(allstats$`gstuae$Fis`)) # There are infinite values messing up the mean calcs
library(data.table)
for (j in 1:ncol(allstats)) set(allstats, which(is.infinite(allstats[[j]])), j, NA)

# Obtain means for each population
allstatspop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=mean, na.rm=TRUE, na.action=NULL)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe")
allstatspop[1:nrow(allstatspop),]

# Calculate number of SNPs used for deriving stats in each population
count_nas <- function(arg1) {
  return(sum(is.na(arg1)))
}
aggregate(allstats[,3:8],by=list(allstats$Stratum),FUN=count_nas)

# Obtain standard deviation for each popn
library("plotrix")
allstatsSDpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=sd, na.rm=TRUE)
colnames(allstatsSDpop) <- c("Pop","ASD","AeSD","HeSD","HoSD","FisSD","PeSD")

# Obtain standard errors for each popn
library("plotrix")
allstatsSEpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=std.error, na.rm=TRUE)
colnames(allstatsSEpop) <- c("Pop","ASE","AeSE","HeSE","HoSE","FisSE","PeSE")

# Cbind SEs to Means
allstatspop <- cbind(allstatspop,
                     allstatsSEpop$ASE, allstatsSEpop$AeSE,allstatsSEpop$HeSE,allstatsSEpop$HoSE,allstatsSEpop$FisSE,allstatsSEpop$PeSE,
                     allstatsSDpop$ASD, allstatsSDpop$AeSD,allstatsSDpop$HeSD,allstatsSDpop$HoSD,allstatsSDpop$FisSD,allstatsSDpop$PeSD)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe",
                           "ASE","AeSE","HeSE","HoSE","FisSE","PeSE",
                           "ASD","AeSD","HeSD","HoSD","FisSD","PeSD")
toadexstats <- allstatspop
toadexstats <- cbind(toadexstats$Pop,round(toadexstats[,2:19],3))
colnames(toadexstats)[1] <- "Locality"

# Output to csv file
filename <- paste("output/popgenstatsEXPOSED",currentDate,".csv",sep="")
write.csv(toadexstats,filename)


## Calcs for island populations
# Convert data to gstudio format
data4gstudio <- genind2df(islands, sep="", usepop=TRUE, oneColPerAll=FALSE)
data4gstudio[data4gstudio==22] <- 2
data4gstudio[data4gstudio==12] <- 1
data4gstudio[data4gstudio==11] <- 0
data4gstudio[1:10,1:10]

# Read into gstudio
write.csv(data4gstudio, "output/gstuddat.csv")
gstudred <- read_population("output/gstuddat.csv", type = "snp", locus.columns = 3:(ncol(data4gstudio)+1))
gstudred[1:10,1:6]

# Group by population
clades <- partition(gstudred, stratum = "pop")
names(clades)
lapply(clades, dim) # looking good

# Get genetic stats by locus for each population
gstua <- genetic_diversity(gstudred, stratum="pop", mode = "A", small.N = TRUE) # This will take ~7 mins
gstuae <- genetic_diversity(gstudred, stratum="pop", mode = "Ae", small.N = TRUE) # This will take ~7 mins
gstuhe <- genetic_diversity(gstudred, stratum="pop", mode = "He", small.N = TRUE) # This will take ~7 mins
gstuho <- genetic_diversity(gstudred, stratum="pop", mode = "Ho", small.N = TRUE) # This will take ~7 mins
gstufis <- genetic_diversity(gstudred, stratum="pop", mode = "Fis", small.N = TRUE) # This will take ~7 mins
gstufpe <- genetic_diversity(gstudred, stratum="pop", mode = "Pe", small.N = TRUE) # This will take ~7 mins
gstuhe[1:30,1:3]
identical(gstuhe[['Locus']],gstufis[['Locus']]) # Good

# Bind all results together
allstats <- cbind(gstua,gstuae$Ae,gstuhe$He,gstuho$Ho,gstufis$Fis,gstufpe$Pe)
allstats[1:10,]

# Change all infinite values to NA
any(is.infinite(allstats$`gstuae$Fis`)) # There are infinite values messing up the mean calcs
library(data.table)
for (j in 1:ncol(allstats)) set(allstats, which(is.infinite(allstats[[j]])), j, NA)

# Obtain means for each population
allstatspop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=mean, na.rm=TRUE, na.action=NULL)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe")
allstatspop[1:nrow(allstatspop),]

# Calculate number of SNPs used for deriving stats in each population
count_nas <- function(arg1) {
  return(sum(is.na(arg1)))
}
aggregate(allstats[,3:8],by=list(allstats$Stratum),FUN=count_nas)

# Obtain standard deviation for each popn
library("plotrix")
allstatsSDpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=sd, na.rm=TRUE)
colnames(allstatsSDpop) <- c("Pop","ASD","AeSD","HeSD","HoSD","FisSD","PeSD")

# Obtain standard errors for each popn
library("plotrix")
allstatsSEpop <- aggregate(allstats[,3:8], by=list(allstats$Stratum), FUN=std.error, na.rm=TRUE)
colnames(allstatsSEpop) <- c("Pop","ASE","AeSE","HeSE","HoSE","FisSE","PeSE")

# Cbind SEs to Means
allstatspop <- cbind(allstatspop,
                     allstatsSEpop$ASE, allstatsSEpop$AeSE,allstatsSEpop$HeSE,allstatsSEpop$HoSE,allstatsSEpop$FisSE,allstatsSEpop$PeSE,
                     allstatsSDpop$ASD, allstatsSDpop$AeSD,allstatsSDpop$HeSD,allstatsSDpop$HoSD,allstatsSDpop$FisSD,allstatsSDpop$PeSD)
colnames(allstatspop) <- c("Pop","A","Ae","He","Ho","Fis","Pe",
                           "ASE","AeSE","HeSE","HoSE","FisSE","PeSE",
                           "ASD","AeSD","HeSD","HoSD","FisSD","PeSD")
islandstats <- allstatspop
islandstats <- cbind(islandstats$Pop,round(islandstats[,2:19],3))
colnames(islandstats)[1] <- "Locality"


# Output to csv file
filename <- paste("output/popgenstatsISLANDS",currentDate,".csv",sep="")
write.csv(islandstats,filename)



# Check em all and combine
islandstats$treatment <- "Island"
toadexstats$treatment <- "Exposed"
toadnaivestats$treatment <- "Naive"
allstatsproper <- rbind(toadnaivestats,toadexstats,islandstats)

# Output to csv file
filename <- paste("output/popgenstatsALL",currentDate,".csv",sep="")
write.csv(allstatsproper,filename)

# Round and output
round_df <- function(x, digits) {
  numeric_columns <- sapply(x, mode) == 'numeric'
  x[numeric_columns] <-  round(x[numeric_columns], digits)
  x
}
allstatsproperround <- round_df(allstatsproper, 3)
filename <- paste("output/popgenstatsALL_round_",currentDate,".csv",sep="")
write.csv(allstatsproperround,filename)







### Check significance of changes through time in Astell Island and East Alligator populations
library(popgraph)
library(gstudio)

## Subset to pop(s) of interest
timepops <- samps[samps$popn=="Astell Island 2005" | samps$popn=="Astell Island 2016"  |
                    samps$popn=="East Alligator 2003" | samps$popn=="East Alligator 2017" , ]

timepops <- genotypes[row.names(genotypes@tab) %in% timepops$sample]
table(timepops@pop)

# Convert data to gstudio format
data4gstudio <- genind2df(timepops, sep="", usepop=TRUE, oneColPerAll=FALSE)
data4gstudio[data4gstudio==22] <- 2
data4gstudio[data4gstudio==12] <- 1
data4gstudio[data4gstudio==11] <- 0
data4gstudio[1:10,1:10]

# Read into gstudio
write.csv(data4gstudio, "output/gstuddat_time_pops.csv")
gstudred <- read_population("output/gstuddat_time_pops.csv", type = "snp", locus.columns = 3:(ncol(data4gstudio)+1))
gstudred[1:10,1:6]

# Group by population
clades <- partition(gstudred, stratum = "pop")
names(clades)
lapply(clades, dim) # looking good

# Get genetic stats by locus for each population
gstua <- genetic_diversity(gstudred, stratum="pop", mode = "A", small.N = TRUE) # Will take ~3 mins
gstuae <- genetic_diversity(gstudred, stratum="pop", mode = "Ae", small.N = TRUE) # Will take ~3 mins
gstuhe <- genetic_diversity(gstudred, stratum="pop", mode = "He", small.N = TRUE) # Will take ~3 mins
gstuho <- genetic_diversity(gstudred, stratum="pop", mode = "Ho", small.N = TRUE) # Will take ~3 mins
gstufis <- genetic_diversity(gstudred, stratum="pop", mode = "Fis", small.N = TRUE) # Will take ~3 mins
gstufpe <- genetic_diversity(gstudred, stratum="pop", mode = "Pe", small.N = TRUE) # Will take ~3 mins
gstuhe[1:30,1:3]
identical(gstuhe[['Locus']],gstufis[['Locus']]) # Good

# Bind all results together
allstats <- cbind(gstua,gstuae$Ae,gstuhe$He,gstuho$Ho,gstufis$Fis,gstufpe$Pe)
allstats[1:10,]

# Change all infinite values to NA
any(is.infinite(allstats$`gstuae$Fis`)) # Check if there are infinite values present
library(data.table)
for (j in 1:ncol(allstats)) set(allstats, which(is.infinite(allstats[[j]])), j, NA) # Remove any infinites
allstats[1:10,]

colnames(allstats)[4:8] <- c("Ae","He","Ho","Fis","Pe")

# Check if there's a significant difference between localities at different time periods
astell <- allstats[allstats$Stratum=="Astell Island 2005" | allstats$Stratum=="Astell Island 2016", ]
eastal <- allstats[allstats$Stratum=="East Alligator 2003" | allstats$Stratum=="East Alligator 2017", ]

boxplot(astell$Fis[astell$Stratum=="Astell Island 2005"])
boxplot(astell$Fis[astell$Stratum=="Astell Island 2016"])

boxplot(eastal$Fis[eastal$Stratum=="East Alligator 2003"])
boxplot(eastal$Fis[eastal$Stratum=="East Alligator 2017"])

hist(astell$He[astell$Stratum=="Astell Island 2005"])
hist(astell$He[astell$Stratum=="Astell Island 2016"])

hist(eastal$He[eastal$Stratum=="East Alligator 2003"])
hist(eastal$He[eastal$Stratum=="East Alligator 2017"])

summary(glm(A~Stratum, data=eastal))
summary(glm(Ae~Stratum, data=eastal))
summary(glm(He~Stratum, data=eastal))
summary(glm(Ho~Stratum, data=eastal))
summary(glm(Fis~Stratum, data=eastal))
summary(glm(Pe~Stratum, data=eastal))

summary(glm(A~Stratum, data=astell))
summary(glm(Ae~Stratum, data=astell))
summary(glm(He~Stratum, data=astell))
summary(glm(Ho~Stratum, data=astell))
summary(glm(Fis~Stratum, data=astell))
summary(glm(Pe~Stratum, data=astell))




############################################################################################
### How well does filtered SNP heterozygosity match/correlate with autosomal heterozygosity?

# read in het values
hetvals <- read.csv("output/hetero_4_correl250321.csv",header=TRUE)
cor.test(hetvals$HeSNP,hetvals$HeAuto)
cor.test(hetvals$HoSNP,hetvals$HoAuto)

# Plot
hetvals <- read.csv("output/hetero_4_ggplot250321.csv",header=TRUE)

library(ggplot2)
yes <- ggplot(data=hetvals) +
  geom_point(aes(x=SNP,y=Autosomal*1000, fill=Parameter, shape=Parameter), colour="black", size = 4) +
  scale_shape_manual(values = c(24,21)) +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black", size = 0.5)) +
  xlab("SNP heterozygosity") + 
  ylab("Autosomal heterozygosity (x 1000)") +
  scale_x_continuous(limits=c(0,0.061) ,breaks=c(0,0.02,0.04,0.06))

yes

# Output
pdffilename <- paste("output/snp_auto_het_plot_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=5, width=6)
yes
dev.off()









##############################################################################
### See if genomic parameter estimates significantly differ between treatments

# Could do expected heterozygosity, observed heterozygosity,
# effective number of alleles, inbreeding coefficient, nucleotide diversity

# Read in data
allstatsproper <- read.csv("output/popgenstats_cleaned_GLM_090322.csv", header=TRUE)
head(allstatsproper)

# Check distribution of variables
hist(allstatsproper$Ae,breaks=10)
hist(allstatsproper$HeSNP,breaks=10)
hist(allstatsproper$HoSNP,breaks=10)
hist(allstatsproper$Fis,breaks=10)
hist(allstatsproper$Pe,breaks=10)


## Get mean values for each treatment
yescat <- allstatsproper
# yescat <- yescat[yescat$Locality!="Astell Island (2005)",]
# yescat <- yescat[yescat$Locality!="East Alligator (2014)",]
yescat <- aggregate(yescat[ ,5:12], list(yescat$Treatment), mean)
yescat <- as.data.frame(cbind(yescat[,1],round(yescat[,2:9],digits=3)))
colnames(yescat)[1] <- "Treatment"
# write.csv(yescat, "output/mean_values_treatment_20220329.csv", row.names = FALSE)


# Prepare data
hets <- allstatsproper
hets <- hets[hets$Treatment!="Translocated",]
hets$Treatment <- as.factor(hets$Treatment)
hets$Locality <- as.factor(hets$Locality)
hets$Region <- as.factor(hets$Region)
# hets <- hets[hets$Locality!="Astell Island (2005)",]
# hets <- hets[hets$Locality!="East Alligator (2014)",]

# Plot scatter of diversity against time since exposure
library(ggplot2)
ggplot(data=hets[c(1:17) , ], aes(x=TimeSinceToad,y=HoSNP)) +
  stat_smooth(method = "lm", col="black") +
  geom_point() +
  xlab("Years since toad invasion") +
  ylab(expression(paste("Observed heterozygosity (",italic(H)*{}[O],")"))) +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5),
        axis.line.y = element_line(color="black", size = 0.5))
# ggsave(filename="output/time_since_toad_plot_lm_2022-03-29.pdf", width=7,height=6)


# Model fit sucks for Fis and Ho; probably stick to He/Ae/Pi
# Build model Using the nlme package to fit a correlation structure
library(nlme) # gls function allows fitting linear model without random effects (required for lme function [e.g. random = ~ 1 | Region])

# Without random effect of region
mod1 <- gls(model = HeSNP ~ Treatment,
            data = hets,
            correlation = corGaus(form = ~ Latitude + Longitude, metric="euclidean", fixed=FALSE))

# check autocorrelation
acf(resid(mod1), lag.max = 15, main = "ACF")

# Check sig
car::Anova(mod1)

# Check model
qqnorm(residuals(mod1))
qqline(residuals(mod1))
hist(residuals(mod1), breaks=20)
plot(Variogram(mod1, form = ~ Latitude + Longitude, resType = "normalized", data = hets))

library(effects)
plot(allEffects(mod1))

# Check contrasts (Tukey multiple comparison)
library(emmeans)
marginal = lsmeans(mod1, ~ Treatment)
pairs(marginal, adjust="tukey")

# # Output results
# sink("output/He_glht_GLM_290322.txt")
# pairs(marginal, adjust="tukey")
# sink()



### Do the same thing for Ae
# Prepare data
aes <- allstatsproper[allstatsproper$Treatment!="Translocated",]
aes$Treatment <- as.factor(aes$Treatment)
aes$Locality <- as.factor(aes$Locality)
aes$Region <- as.factor(aes$Region)
# aes <- aes[aes$Locality!="Astell Island (2005)",]
# aes <- aes[aes$Locality!="East Alligator (2014)",]

# Build model Using the nlme package to fit a correlation structure
library(nlme) # gls function allows fitting linear model without random effects (required for lme function [e.g. random = ~ 1 | Region])
mod2 <- gls(model = Ae ~ Treatment,
            data = aes,
            correlation = corGaus(form = ~ Latitude + Longitude, metric="euclidean", fixed=FALSE))

# Check sig
car::Anova(mod2)

# check autocorrelation
acf(resid(mod2), lag.max = 15, main = "ACF")

# Check model
car::Anova(mod2)
qqnorm(residuals(mod2))
qqline(residuals(mod2))
hist(residuals(mod2), breaks=15)
plot(Variogram(mod2, form = ~ Latitude + Longitude, resType = "normalized", data = aes))

library(effects)
plot(allEffects(mod2))

# Check contrasts (Tukey multiple comparison)
library(emmeans)
marginal = lsmeans(mod2,~ Treatment)
pairs(marginal, adjust="tukey")

# # Output results
# sink("output/Ae_glht_GLM_290322.txt")
# pairs(marginal, adjust="tukey")
# sink()



### Do the same thing for Fis
# Prepare data
fiss <- allstatsproper[allstatsproper$Treatment!="Translocated",]
fiss$Treatment <- as.factor(fiss$Treatment)
fiss$Locality <- as.factor(fiss$Locality)
fiss$Region <- as.factor(fiss$Region)
# fiss <- fiss[fiss$Locality!="Astell Island (2005)",]
# fiss <- fiss[fiss$Locality!="East Alligator (2014)",]

# Build model Using the nlme package to fit a correlation structure
library(nlme) # gls function allows fitting linear model without random effects (required for lme function [e.g. random = ~ 1 | Region])
mod3 <- gls(model = Fis ~ Treatment,
            data = fiss,
            correlation = corGaus(form = ~ Latitude + Longitude, metric="euclidean", fixed=FALSE))

# Check sig
car::Anova(mod3)

# check autocorrelation
acf(resid(mod3), lag.max = 15, main = "ACF")

# Check model
car::Anova(mod3)
qqnorm(residuals(mod3))
qqline(residuals(mod3))
hist(residuals(mod3), breaks=15)
plot(Variogram(mod3, form = ~ Latitude + Longitude, resType = "normalized", data = fiss))

library(carData)
library(effects)
plot(allEffects(mod3))

# Check contrasts (Tukey multiple comparison)
library(emmeans)
marginal = lsmeans(mod3,~ Treatment)
pairs(marginal, adjust="tukey")

# # Output results
# sink("output/FIS_glht_GLM_290322.txt")
# pairs(marginal, adjust="tukey")
# sink()



### Do the same thing for Pe
# Prepare data
pes <- allstatsproper[allstatsproper$Treatment!="Translocated",]
pes$Treatment <- as.factor(pes$Treatment)
pes$Locality <- as.factor(pes$Locality)
pes$Region <- as.factor(pes$Region)
# pes <- pes[pes$Locality!="Astell Island (2005)",]
# pes <- pes[pes$Locality!="East Alligator (2014)",]

# Build model Using the nlme package to fit a correlation structure
library(nlme) # gls function allows fitting linear model without random effects (required for lme function [e.g. random = ~ 1 | Region])
mod4 <- gls(model = Pe ~ Treatment,
            data = pes,
            correlation = corGaus(form = ~ Latitude + Longitude, metric="euclidean", fixed=FALSE))

# Check sig
car::Anova(mod4)

# check autocorrelation
acf(resid(mod4), lag.max = 15, main = "ACF")

# Check model
car::Anova(mod4)
qqnorm(residuals(mod4))
qqline(residuals(mod4))
hist(residuals(mod4), breaks=15)
plot(Variogram(mod4, form = ~ Latitude + Longitude, resType = "normalized", data = pes))

library(carData)
library(effects)
plot(allEffects(mod4))

# Check contrasts (Tukey multiple comparison)
library(emmeans)
marginal = lsmeans(mod4,~ Treatment)
pairs(marginal, adjust="tukey")

# # Output results
# sink("output/Pe_glht_GLM_290322.txt")
# pairs(marginal, adjust="tukey")
# sink()



### Do the same thing for Ho
# Prepare data
hots <- allstatsproper
hots <- hots[hots$Treatment!="Translocated",]
hots$Treatment <- as.factor(hots$Treatment)
hots$Locality <- as.factor(hots$Locality)
hots$Region <- as.factor(hots$Region)
# hots <- hots[hots$Locality!="Astell Island (2005)",]
# hots <- hots[hots$Locality!="East Alligator (2014)",]

# Model fit sucks for Fis and Ho; probably stick to He/Ae/Pi
# Build model Using the nlme package to fit a correlation structure
library(nlme) # gls function allows fitting linear model without random effects (required for lme function [e.g. random = ~ 1 | Region])

# Without random effect of region
mod5 <- gls(model = HoSNP ~ Treatment,
            data = hots,
            correlation = corGaus(form = ~ Latitude + Longitude, metric="euclidean", fixed=FALSE))

# check autocorrelation
acf(resid(mod5), lag.max = 15, main = "ACF")

# Check sig
car::Anova(mod5)

# Check model
qqnorm(residuals(mod5))
qqline(residuals(mod5))
hist(residuals(mod5), breaks=20)
plot(Variogram(mod5, form = ~ Latitude + Longitude, resType = "normalized", data = hots))

library(effects)
plot(allEffects(mod5))

# Check contrasts (Tukey multiple comparison)
library(emmeans)
marginal = lsmeans(mod5, ~ Treatment)
pairs(marginal, adjust="tukey")

# Output results
sink("output/Ho_glht_GLM_290322.txt")
pairs(marginal, adjust="tukey")
sink()











### Plot results
# Prepare data frame for plotting
library(ggeffects)
Heeffect <- ggpredict(mod1)
Heeffect <- data.frame(Heeffect$Treatment$x,Heeffect$Treatment$predicted,Heeffect$Treatment$conf.low,Heeffect$Treatment$conf.high)
colnames(Heeffect) <- c("Treatment","Value","Lower","Upper")
Aeeffect <- ggpredict(mod2)
Aeeffect <- data.frame(Aeeffect$Treatment$x,Aeeffect$Treatment$predicted,Aeeffect$Treatment$conf.low,Aeeffect$Treatment$conf.high)
colnames(Aeeffect) <- c("Treatment","Value","Lower","Upper")
Fiseffect <- ggpredict(mod3)
Fiseffect <- data.frame(Fiseffect$Treatment$x,Fiseffect$Treatment$predicted,Fiseffect$Treatment$conf.low,Fiseffect$Treatment$conf.high)
colnames(Fiseffect) <- c("Treatment","Value","Lower","Upper")
Peseffect <- ggpredict(mod4)
Peseffect <- data.frame(Peseffect$Treatment$x,Peseffect$Treatment$predicted,Peseffect$Treatment$conf.low,Peseffect$Treatment$conf.high)
colnames(Peseffect) <- c("Treatment","Value","Lower","Upper")
Hoeffect <- ggpredict(mod5)
Hoeffect <- data.frame(Hoeffect$Treatment$x,Hoeffect$Treatment$predicted,Hoeffect$Treatment$conf.low,Hoeffect$Treatment$conf.high)
colnames(Hoeffect) <- c("Treatment","Value","Lower","Upper")


alleffs <- rbind(Heeffect,Aeeffect,Fiseffect,Peseffect, Hoeffect)
alleffs$Parameter <- c(rep("He",3),rep("Ae",3), rep("Fis",3), rep("Pe",3), rep("Ho",3))



## Custom facet wrap scale functions (because setting the y-axis breaks on facet_wrap sucks hard)
scale_override <- function(which, scale) {
  if(!is.numeric(which) || (length(which) != 1) || (which %% 1 != 0)) {
    stop("which must be an integer of length 1")
  }
  
  if(is.null(scale$aesthetics) || !any(c("x", "y") %in% scale$aesthetics)) {
    stop("scale must be an x or y position scale")
  }
  
  structure(list(which = which, scale = scale), class = "scale_override")
}


CustomFacetWrap <- ggproto(
  "CustomFacetWrap", FacetWrap,
  init_scales = function(self, layout, x_scale = NULL, y_scale = NULL, params) {
    # make the initial x, y scales list
    scales <- ggproto_parent(FacetWrap, self)$init_scales(layout, x_scale, y_scale, params)
    
    if(is.null(params$scale_overrides)) return(scales)
    
    max_scale_x <- length(scales$x)
    max_scale_y <- length(scales$y)
    
    # ... do some modification of the scales$x and scales$y here based on params$scale_overrides
    for(scale_override in params$scale_overrides) {
      which <- scale_override$which
      scale <- scale_override$scale
      
      if("x" %in% scale$aesthetics) {
        if(!is.null(scales$x)) {
          if(which < 0 || which > max_scale_x) stop("Invalid index of x scale: ", which)
          scales$x[[which]] <- scale$clone()
        }
      } else if("y" %in% scale$aesthetics) {
        if(!is.null(scales$y)) {
          if(which < 0 || which > max_scale_y) stop("Invalid index of y scale: ", which)
          scales$y[[which]] <- scale$clone()
        }
      } else {
        stop("Invalid scale")
      }
    }
    
    # return scales
    scales
  }
)

facet_wrap_custom <- function(..., scale_overrides = NULL) {
  # take advantage of the sanitizing that happens in facet_wrap
  facet_super <- facet_wrap(...)
  
  # sanitize scale overrides
  if(inherits(scale_overrides, "scale_override")) {
    scale_overrides <- list(scale_overrides)
  } else if(!is.list(scale_overrides) || 
            !all(vapply(scale_overrides, inherits, "scale_override", FUN.VALUE = logical(1)))) {
    stop("scale_overrides must be a scale_override object or a list of scale_override objects")
  }
  
  facet_super$params$scale_overrides <- scale_overrides
  
  ggproto(NULL, CustomFacetWrap,
          shrink = facet_super$shrink,
          params = facet_super$params
  )
}


# Make plot
library(ggplot2)
library(scales)
alleffsplot <- ggplot(alleffs, aes(x=Treatment, y=Value)) +
  geom_point(stat = "identity", position = position_dodge(width=0.4), size=2) +
  geom_errorbar(aes(ymin = Lower, ymax = Upper), alpha = 0.9, width=0.1, position = position_dodge(width=0.4)) +
  facet_wrap_custom(~Parameter, scales = "free_y", nrow=5, ncol = 1, scale_overrides = list(
    scale_override(1, scale_y_continuous(limits=c(1.01,1.115), breaks = c(1.03, 1.07, 1.11))),
    scale_override(2, scale_y_continuous(limits=c(-0.04,0.105), breaks = c(-0.02, 0.04, 0.1))),
    scale_override(3, scale_y_continuous(limits=c(0.005,0.085), breaks = c(0.01,0.04,0.07))),
    scale_override(4, scale_y_continuous(limits=c(0.005,0.085), breaks = c(0.01,0.04,0.07))),
    scale_override(5, scale_y_continuous(limits=c(0.005,0.085), breaks = c(0.01,0.04,0.07))))) +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, colour="black")) +
  scale_y_continuous(breaks = scales::breaks_extended(n = 5))

alleffsplot



# Output
pdffilename <- paste("output/AeFisHeHoPe_plot_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=10, width=4)
alleffsplot
dev.off()

## Add p-values to plot manually in inkscape








###############################
# Get pairwise FST/GST matrix #
###############################

# And check significance of pairwise FST between populations

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
samps$region[samps$popn=="Astell Island 2016"] <- "English Company Islands"
samps$region[samps$popn=="Koolan Island"] <- "Kimberley"
samps$popn[samps$popn=="Mornington Station"] <- "Mornington Sanctuary"
samps$popn[samps$popn=="King Leopold West"] <- "Wunaamin Miliwundi West"
samps$popn[samps$popn=="Northwest Marble Bar"] <- "Marble Bar"
samps$popn[samps$popn=="Millstream Chichester"] <- "Millstream Chichester NP"

# Check some popn size stuff
poptab <- as.data.frame(table(samps$popn))
nrow(poptab[poptab$Freq<6,])
nrow(poptab[poptab$Freq<8,])
samps <- samps[!samps$popn %in% poptab[poptab$Freq<6,]$Var1 , ] # Remove small populations
# samps <- samps[!samps$popn == "Astell Island 2005" , ]
# samps <- samps[!samps$popn == "Darwin 2000" , ]
# samps <- samps[!samps$popn == "East Alligator 2003" , ]
# samps <- samps[!samps$popn == "King Leopold East" , ]
# samps <- samps[!samps$popn == "Pobassoo Island" , ]
length(unique(samps$popn))

# Rearrange rows so it's state -> region -> popn -> sample
library(dplyr)
samps <- samps %>% arrange(state, region, popn)

# Make sure genetic data is identical to sample metadata
identical(samps$sample,rownames(dasgen))
dasgen <- subset(dasgen, rownames(dasgen) %in% samps$sample)
dasgen <- dasgen[order(match(rownames(dasgen), samps$sample)), ]
identical(rownames(dasgen),samps$sample)
# samps$popn <- gsub(" ", "", samps$popn) # Removes spaces from population names - not necessary

# ## Use the strataG package which provides tests for gst.dbl.prime (NOT WORKING)
# library(strataG)
# # Convert to stupid gtypes format
# dasgtype <- as.data.frame(cbind(rownames(dasgen),samps$popn,dasgen[,1:ncol(dasgen)]))
# rownames(dasgtype) <- c(1:nrow(dasgtype))
# colnames(dasgtype)[1] <- "id"
# colnames(dasgtype)[2] <- "popn"
# dasgtype[1:10,1:10]
# dasgtype <- df2gtypes(dasgtype, ploidy = 1, id.col = 1, strata.col = 2, loc.col = 3)
# library(dplyr)
# overallTest(dasgtype, nrep = 3, by.locus=FALSE, stats = c("fst", "gst.dbl.prime"), quietly = FALSE, num.cores = 6) # Not working
# pairwiseTest(dasgtype, nrep = 3, by.locus=FALSE, stats = "gst.dbl.prime", quietly = FALSE, max.cores = 6) # Not working

# Convert data to right format for genind
dasgen[dasgen==2] <- 22
dasgen[dasgen==1] <- 12
dasgen[dasgen==0] <- 11
dasgen[1:10,1:10] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(dasgen, ploidy=2, pop=samps$popn, ncode=1)  # Takes a while with large datasets
genotypes

# # Run diff_test using the mmod package
# library(mmod)
# diff_test(genotypes, sim = FALSE, nreps = 3) # Not running due to stupid missing slot in stupid genind object

# # # Can calc differentiation with hierfstat package
# library(hierfstat)
# neif <- pairwise.neifst(data.frame(samps$popn,dasgen), diploid=TRUE) # Takes ages, no significance values

# This is really f**kin tiresome

## Ok so I'm now going to calculate G''ST to put into the supplementary material
## calc G''ST and classical Fst and compare them using a mantel test
## We present the classical values in the main document because they're fine for
## biallelic SNPs and they are easy to get p-values

library(mmod) # Modern Methods of Differentiation
popgst <- pairwise_Gst_Hedrick(genotypes) # Calculates pairwise population values of G''st - TAKES AN HOUR
popgst <- as.data.frame(as.matrix(popgst))
popgst[upper.tri(popgst, diag=FALSE)] <- 0

# Can use the stamppfst method to get p values - this is for Weir and Cockerham 1984 Fst - TAKES AN HOUR
library(StAMPP)
library(dartR)
genotypegl <- gi2gl(genotypes)
sigvals <- stamppFst(genotypegl, nboots = 1000, percent = 95, nclusters = 10)
p.adjust(sigvals$Pvalues, method = "fdr")
popfst <- as.data.frame(as.matrix(sigvals$Fsts))
popfst[upper.tri(popfst, diag=FALSE)] <- 0


## Output all results
write.csv(popgst,"output/population_Gst_090322.csv",row.names=TRUE)
write.csv(popfst,"output/population_Fst_090322.csv",row.names=TRUE)
write.csv(as.data.frame(as.matrix(sigvals$Pvalues)),"output/population_Fst_pVals_090322.csv",row.names=TRUE)



# Run mantel test
fst <- read.csv("output/population_Fst_090322.csv",row.names = 1,header=TRUE)
gst <- read.csv("output/population_Gst_090322.csv",row.names = 1,header=TRUE)
diag(fst) <- diag(gst) <- 0
manres <- mantel.test(gst,fst,nperm=9999)



## Visualise Fst in a (somewhat) creative manner
fst <- read.csv("output/population_Fst_090322.csv",row.names = 1,header=TRUE)
fst <- as.dist(fst)
dst <- data.matrix(fst)
dim <- ncol(dst)

### Output heatmap to pdf

# Get lower triangle of the correlation matrix
get_lower_tri<-function(cormat){
  cormat[upper.tri(cormat)] <- NA
  return(cormat)
}
heaty <- get_lower_tri(dst)

# Convert to stupid ggplot format
library(reshape2)
heaty <- melt(heaty, na.rm=TRUE)
head(heaty)

## Plot heatmap using ggplot2
library(ggplot2)
heatmappy <- ggplot(heaty, aes(Var1, Var2)) +
  geom_tile(aes(fill = value)) +
  scale_fill_gradient2(low = "#fcffa4", mid="#b73779", high = "#440154", 
                       midpoint = 0.5, limit = c(0,1), space = "Lab", 
                       name=expression(italic(F)[ST])) +
  theme_bw()+
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_blank(),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x = element_text(angle = 45, vjust = 1, size = 12, hjust = 1),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5),
        axis.line.y = element_line(color="black", size = 0.5),
        legend.position = c(0.85, 0.7)) +
  coord_fixed() +
  scale_x_discrete(limits = rev(levels(heaty$Var1)), expand=c(0,0)) +
  scale_y_discrete(expand=c(0,0)) +
  guides(fill = guide_colourbar(barwidth = 0.7,
                                barheight = 20)) +
  geom_text(aes(Var1, Var2, label = round(value,1)), color = "black", size = 4)

heatmappy


# Output FST heatmap plot
pdffilename <- paste("output/fst_plot_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=10, height=10)
heatmappy
dev.off()




## Visualise GST in a (somewhat) creative manner
gst <- read.csv("output/population_Gst_090322.csv",row.names = 1,header=TRUE)
gst <- as.dist(gst)
dst <- data.matrix(gst)
dim <- ncol(dst)

### Output heatmap to pdf

# Get lower triangle of the correlation matrix
get_lower_tri<-function(cormat){
  cormat[upper.tri(cormat)] <- NA
  return(cormat)
}
heaty <- get_lower_tri(dst)

# Convert to stupid ggplot format
library(reshape2)
heaty <- melt(heaty, na.rm=TRUE)
head(heaty)

## Plot heatmap using ggplot2
library(ggplot2)
heatmappy <- ggplot(heaty, aes(Var1, Var2)) +
  geom_tile(aes(fill = value)) +
  scale_fill_gradient2(low = "#fcffa4", mid="#b73779", high = "#440154", 
                       midpoint = 0.5, limit = c(0,1), space = "Lab", 
                       name=expression(italic(G)[ST])) +
  theme_bw()+
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_blank(),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x = element_text(angle = 45, vjust = 1, size = 12, hjust = 1),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5),
        axis.line.y = element_line(color="black", size = 0.5),
        legend.position = c(0.85, 0.7)) +
  coord_fixed() +
  scale_x_discrete(limits = rev(levels(heaty$Var1)), expand=c(0,0)) +
  scale_y_discrete(expand=c(0,0)) +
  guides(fill = guide_colourbar(barwidth = 0.7,
                                barheight = 20)) +
  geom_text(aes(Var1, Var2, label = round(value,1)), color = "black", size = 4)

heatmappy


# Output FST heatmap plot
pdffilename <- paste("output/gst_plot_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=10, height=10)
heatmappy
dev.off()










############################################################
# Do spatial autocorrelation analysis using GenAlEx method #
############################################################

library(gstudio)

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))

# Remove samps without coordinates
samps <- samps[!is.na(samps$lat), ]
dasgen <- subset(dasgen, rownames(dasgen) %in% samps$sample)
popns <- samps$popn
identical(samps$sample,rownames(dasgen))

geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Remove island populations
unique(genotypes@pop)
islandpops <- c("Groote East","Groote South","Groote West","Pellew Islands","Astell Island","Koolan Island","Dolphin Island", "Marchinbar Island", "Pobassoo Island")

for(i in islandpops){
  genotypes <- genotypes[genotypes@pop!=i,]
}

# Get genetic distance matrix
library(poppr)
gendist <- bitwise.dist(genotypes, percent = TRUE, mat = FALSE)
gendist <- as.matrix(gendist)

# Get geographic distance matrix
samps <- subset(samps, samps$sample %in% rownames(genotypes@tab))

# Check that they are in the same order as the rownames of genotypes
identical(samps$sample,rownames(genotypes@tab))

library(fossil)
geogcoords <- samps[c("long", "lat")]
geogdist <- earth.dist(geogcoords,dist=TRUE) # This is in km
geogdist <- as.matrix(geogdist)
colnames(geogdist) <- rownames(geogdist) <- rownames(genotypes@tab)

# Run autocorrelation analysis
genauto <- genetic_autocorrelation(P=geogdist, G=gendist, bins=c(0,1,5,50,100,500,1000,5000), perms=999, plot=TRUE)

# Save to file
write.csv(genauto,"output/spatial_autocor_250322.csv",row.names = FALSE)

# Make plot
library(ggplot2)
library(ggrepel)
corrgram <- ggplot(data=genauto, aes(x=as.factor(To), y=R)) +
  geom_point(aes(), size=1.3) +
  geom_line(group=1, aes(), size=0.5) +
  geom_text_repel(aes(label=paste("P = ",round(P,digits=3),sep="")), force=1, point.padding=0.2, segment.color = "transparent") +
  geom_hline(yintercept = 0, size = 0.3, colour = "black", linetype = "dotted") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank()) +  
  theme(panel.grid.minor.y = element_blank()) +
  theme(panel.grid.major.x = element_blank()) +  
  theme(panel.grid.minor.x = element_blank()) +
  xlab("End point of distance class (km)") + 
  ylab("Spatial autocorrelation") +
  theme(axis.title.y=element_text(margin=margin(0,10,0,0), colour = "black")) +
  theme(axis.title.x=element_text(margin=margin(10,0,0,0), colour = "black")) +
  theme(axis.text.x = element_text(colour="black")) +
  theme(axis.text.y = element_text(colour="black")) +
  theme(panel.border = element_blank()) +
  theme(legend.position = c(0.8, 0.8)) +
  theme(axis.line.x = element_line(color="black", size = 0.5), axis.line.y = element_line(color="black",
                                                                                          size = 0.5))

corrgram

# Output spatial autocorrelogram plot
pdffilename <- paste("output/dasy_spatial_structure_mainland_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=7, height=4)
corrgram
dev.off()








##################################################
# Do scatter plot of geog dist by population fst #
##################################################

library(gstudio)

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))

# Remove samps without coordinates
samps <- samps[!is.na(samps$lat), ]
dasgen <- subset(dasgen, rownames(dasgen) %in% samps$sample)
popns <- samps$popn
identical(samps$sample,rownames(dasgen))

geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Remove island populations
unique(genotypes@pop)
islandpops <- c("Groote East","Groote South","Groote West","Pellew Islands","Astell Island","Koolan Island","Dolphin Island", "Marchinbar Island", "Pobassoo Island")

for(i in islandpops){
  genotypes <- genotypes[genotypes@pop!=i,]
}

# Remove small populations
smallpops <- as.data.frame(table(popns))[as.data.frame(table(popns))$Freq<6,]
smallpops <- as.character(smallpops$popns)
for(i in smallpops){
  genotypes <- genotypes[genotypes@pop!=i,]
}

# Use StAMPP to get Weir and Cockerham 1984 FST values among populations
library(StAMPP)
library(dartR)
library(poppr)
genotypegl <- gi2gl(genotypes)
sigvals <- stamppFst(genotypegl, nboots = 1, nclusters = 10)
popfst <- as.data.frame(as.matrix(sigvals))
popfst[upper.tri(popfst)]<-t(popfst)[upper.tri(popfst)]

# Get geographic distance matrix
samps <- subset(samps, samps$sample %in% rownames(genotypes@tab))

# Check that they are in the same order as the rownames of genotypes
identical(samps$sample,rownames(genotypes@tab))

# Obtain mean values for lat and long
library(fossil)
geogcoords <- aggregate(samps[, 5:6], list(samps$popn), mean)
rownames(popfst)
geogcoords <- geogcoords[order(match(geogcoords$Group.1, rownames(popfst))), ]
geogcoords <- geogcoords[c("long", "lat")]
geogdist <- earth.dist(geogcoords,dist=TRUE) # This is in km
geogdist <- as.matrix(geogdist)
colnames(geogdist) <- rownames(geogdist) <- rownames(popfst)

# Check mantel test
gendy <- popfst
geogy <- geogdist
mantel.test(gendy,geogy, nperm = 9999) # z.stat = 38952.34 p = 0.0001

## Make plot of log geog dist by genetic dist
gendy[upper.tri(gendy, diag=TRUE)] <- NA
geogy[upper.tri(geogy, diag=TRUE)] <- NA

library(reshape2)
gendy <- melt(as.matrix(gendy), varnames = c("row", "col"))
geogy <- melt(as.matrix(geogy), varnames = c("row", "col"))
identical(geogy$row,gendy$row)
identical(geogy$col,gendy$col)
df <- cbind(gendy$row,gendy$col,gendy$value,geogy$value)
colnames(df) <- c("row","col","FST","GeogDist")
df <- df[complete.cases(df), ]
df <- as.data.frame(df)
# df$GeogDist <- log(df$GeogDist)
# df$GeogDist[!is.finite(df$GeogDist)] <- 0
df$FST <- df$FST/(1-df$FST)


library(ggplot2)
fstgeogplot <- ggplot(data=df, (aes(x=GeogDist, y=FST))) +
  geom_point() +
  xlab("Geographic distance (km)") +
  ylab(expression(italic(F)[ST]/(1-italic(F)[ST]))) + 
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black"),
        panel.border = element_blank(),
        axis.line.x = element_line(color="black", size = 0.5),
        axis.line.y = element_line(color="black", size = 0.5)) +
  geom_smooth(method="glm",colour="black", linetype=1, size=0.4) +
  scale_y_continuous(limits=c(0,1.5))


# Output pairwise geographic distance by fst plot
pdffilename <- paste("output/dasy_fst_geog_dist_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=6, height=5)
fstgeogplot
dev.off()

summary(glm(FST ~ GeogDist, data=df), )







###########################################################
# Check population structure and map results using tess3r #
###########################################################

# See Caye et al. 2016 and 2018 for details of algorithms

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
popns <- samps$popn
samps$region[samps$popn=="Astell Island"] <- "English Company Islands"

# Impute missing values in coordinates
samps[5:6] <- sapply(samps[5:6], function(x){
  ave(x, samps[[4]], FUN = function(.x) {
    .x[is.na(.x)] <- mean(.x, na.rm = TRUE)
    .x
  })
})



# tess3r package newer and better than LEA
#devtools::install_github("bcm-uga/TESS3_encho_sen")
library("tess3r")

## Prepare data
dasgen[1:10,1:10]
Y <- dasgen

## Create tess3r project
# Need lat and longs
latlongs <- as.matrix(samps[,5:6])
latlongs <- latlongs[,c(2,1)]

# set up allinfo for later
yeahcats <- aggregate(samps[ ,5:6], list(samps$popn), mean)


# Runs (latent factors = ancestral populations??) K = 1:x with cross-entropy calcs, each done with y
# repetitions.
tess3.obj <- tess3(X = Y, coord = latlongs, K = 1:15, rep=10, # including rep just puts error bars onto scree plot
                   method = "projected.ls", ploidy = 2, openMP.core.num = 8)

# plot cross-entropy criterion of all runs of the project
plot(tess3.obj, pch = 19, col = "blue",
     xlab = "Number of ancestral populations",
     ylab = "Cross-validation score")


library(dplyr)
library(reshape)
library(ggplot2)

## Make admixture plot using K=2
# take the ancestry coefficients for the best run where K = 2
qmatrix2 <- qmatrix(tess3.obj, K = 2)

# format for ggplot
qmatrix2 <- as.data.frame(qmatrix2)
qmatrix2$popn <- samps$popn
qmatrix2$sampname <- rownames(Y)
qmatrix2$region <- samps$region
qmatrix2$state <- samps$state
qmatrix2 <- qmatrix2 %>% arrange(state, region, popn)
qmatrix2$state <- NULL
qmatrix2 <- melt(qmatrix2,id.vars=c("popn","sampname","region"))
qmatrix2$sampname <- as.factor(qmatrix2$sampname)
qmatrix2$sampname <- factor(qmatrix2$sampname, levels = qmatrix2$sampname[1:length(rownames(Y))])


# Make plot for k2
q2 <- ggplot(qmatrix2, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#E69F00","#56B4E9","#009E73","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix2$popn)


## Make admixture plot using K=3
# get the cross-entropy of each run
qmatrix3 <- qmatrix(tess3.obj, K = 3)
# format for ggplot
qmatrix3 <- as.data.frame(qmatrix3)
qmatrix3$popn <- popns
qmatrix3$sampname <- rownames(Y)
qmatrix3$region <- samps$region
qmatrix3$state <- samps$state
qmatrix3 <- qmatrix3 %>% arrange(state, region, popn)
qmatrix3$state <- NULL
qmatrix3 <- melt(qmatrix3,id.vars=c("popn","sampname","region"))
qmatrix3$sampname <- as.factor(qmatrix3$sampname)
qmatrix3$sampname <- factor(qmatrix3$sampname, levels = qmatrix3$sampname[1:length(rownames(Y))])

# make plot for K3
q3 <- ggplot(qmatrix3, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#56B4E9","#E69F00","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text=element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix3$popn)



## Make admixture plot using K=4
# get the cross-entropy of each run
qmatrix4 <- qmatrix(tess3.obj, K = 4)
# format for ggplot
qmatrix4 <- as.data.frame(qmatrix4)
qmatrix4$popn <- popns
qmatrix4$sampname <- rownames(Y)
qmatrix4$region <- samps$region
qmatrix4$state <- samps$state
qmatrix4 <- qmatrix4 %>% arrange(state, region, popn)
qmatrix4$state <- NULL
qmatrix4 <- melt(qmatrix4,id.vars=c("popn","sampname","region"))
qmatrix4$sampname <- as.factor(qmatrix4$sampname)
qmatrix4$sampname <- factor(qmatrix4$sampname, levels = qmatrix4$sampname[1:length(rownames(Y))])

# make plot for K4
q4 <- ggplot(qmatrix4, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#56B4E9","#009E73","#E69F00","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text=element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix4$popn)



## Make admixture plot using K=5
# get the cross-entropy of each run
qmatrix5 <- qmatrix(tess3.obj, K = 5)
# format for ggplot
qmatrix5 <- as.data.frame(qmatrix5)
qmatrix5$popn <- popns
qmatrix5$sampname <- rownames(Y)
qmatrix5$region <- samps$region
qmatrix5$state <- samps$state
qmatrix5 <- qmatrix5 %>% arrange(state, region, popn)
qmatrix5[order(qmatrix5[,5], qmatrix5[,3]), ]
qmatrix5$state <- NULL
qmatrix5 <- melt(qmatrix5,id.vars=c("popn","sampname","region"))
qmatrix5$sampname <- as.factor(qmatrix5$sampname)
qmatrix5$sampname <- factor(qmatrix5$sampname, levels = qmatrix5$sampname[1:length(rownames(Y))])

# make plot for K5
q5 <- ggplot(qmatrix5, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#E69F00","#56B4E9","#009E73","#FF0099","#F0E442")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text=element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix5$popn)


## Make admixture plot using K=6
# get the cross-entropy of each run
qmatrix6 <- qmatrix(tess3.obj, K = 6)
# format for ggplot
qmatrix6 <- as.data.frame(qmatrix6)
qmatrix6$popn <- popns
qmatrix6$sampname <- rownames(Y)
qmatrix6$region <- samps$region
qmatrix6$state <- samps$state
qmatrix6 <- qmatrix6 %>% arrange(state, region, popn)
qmatrix6$state <- NULL
qmatrix6 <- melt(qmatrix6,id.vars=c("popn","sampname","region"))
qmatrix6$sampname <- as.factor(qmatrix6$sampname)
qmatrix6$sampname <- factor(qmatrix6$sampname, levels = qmatrix6$sampname[1:length(rownames(Y))])


# make plot for K6
q6 <- ggplot(qmatrix6, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#009E73","#FF0099","#E69F00","#0072b2","#F0E442","#56B4E9")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        axis.text=element_text(size = 13, color="black"),
        axis.text.x=element_blank()) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix6$popn)


## Make admixture plot using K=7
# get the cross-entropy of each run
qmatrix7 <- qmatrix(tess3.obj, K = 7)
# format for ggplot
qmatrix7 <- as.data.frame(qmatrix7)
qmatrix7$popn <- popns
qmatrix7$sampname <- rownames(Y)
qmatrix7$region <- samps$region
qmatrix7$state <- samps$state
qmatrix7 <- qmatrix7 %>% arrange(state, region, popn)
qmatrix7$state <- NULL
qmatrix7 <- melt(qmatrix7,id.vars=c("popn","sampname","region"))
qmatrix7$sampname <- as.factor(qmatrix7$sampname)
qmatrix7$sampname <- factor(qmatrix7$sampname, levels = qmatrix7$sampname[1:length(rownames(Y))])


# make plot for K7
q7 <- ggplot(qmatrix7, aes(fill=variable, y=value, x=sampname)) + 
  geom_bar(position="stack", stat="identity", width=1) +
  scale_fill_manual(values=c("#56B4E9","#FF0099","#F0E442","#0072b2","#db1640","#E69F00","#009E73")) +
  xlab("Population") +
  ylab("Admixture coefficients") + 
  theme_bw() +
  guides(fill=FALSE) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black"),
        axis.text.x=element_text(angle = 90, size = 8)) +
  scale_y_continuous(expand=c(0,0)) +
  scale_x_discrete(expand=c(0,0), labels=qmatrix7$popn)## Make admixture plot using K=5


### output plots K 2,3,4,5

# Organise layout of plots
lay <- rbind(c(1,1,1),
             c(1,1,1),
             c(1,1,1),
             c(1,1,1),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(3,3,3),
             c(3,3,3),
             c(3,3,3),
             c(3,3,3),
             c(4,4,4),
             c(4,4,4),
             c(4,4,4),
             c(4,4,4),
             c(5,5,5),
             c(5,5,5),
             c(5,5,5),
             c(5,5,5),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6),
             c(6,6,6))

library(gridExtra)
pdffilename <- paste("output/tess3r_admix_plots_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=9, width=20)
grid.arrange(q2, q3, q4, q5, q6, q7, layout_matrix=lay)
dev.off()

###





###### Map the k values across the landscape

###

#install.packages("rworldmap")
#install.packages("rworldxtra")
library(rworldmap)
library(rworldxtra)
library(ggplot2)
library(ggsn)

# Get map outline to use
map.polygon <- getMap(resolution = "high")
map.polygon <- map.polygon[map.polygon$SOVEREIGNT=="Australia",]
map.polygon <- map.polygon[map.polygon$ADMIN=="Australia",]

library(sf)
dsn <- "output"
lay <- "Dasy_hal_hist_dist_BvT_20220204"
histdist <- read_sf(dsn = dsn, layer = lay) # Read in coastline


## K = 2
# Prepare q matrix for output
qmat <- as.data.frame(qmatrix(tess3.obj, K = 2))
rownames(qmat) <- rownames(dasgen)

## Use CreatePalette() to define color palettes.
my.colors <- c("#E69F00","#56B4E9","#FF0099","#F0E442","#009E73", "tan1", "red")
my.palette <- CreatePalette(my.colors, 10)

# Make high resolution map
k2 <- ggtess3Q(qmat, latlongs, map.polygon=histdist, col.palette=my.palette,
                         interpolation.model = FieldsKrigModel(10),
                         resolution = c(1200,1200)) +
  geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
  xlim(113, 147) + 
  ylim(-22.5,-10) + 
  coord_equal() + 
  geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
  xlab("Longitude") +
  ylab("Latitude") + 
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        # axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text.x = element_blank(),
        # axis.text.x = element_text(size = 14, color="black"),
        axis.text.y = element_text(size = 14, color="black")) +
  # ggsn::scalebar(location="topleft",y.min=-22.5, y.max=-11, x.min=113, x.max=147, anchor=c(x=127.4,y=-22), 
  #        dist=500, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.09, st.size=4, height = 0.0365) +
  north(location = "topright", scale = 0.2, symbol = 1,
        y.min=-22.5, y.max=-10, x.min=113, x.max=147, anchor = c(x=150,y=-10.2)) +
  annotate('text', 116.5, -11.25, label=paste("italic(k)==", 2, sep=""), parse=TRUE, hjust=1, size=8) +
  coord_quickmap()



## K = 3
# Prepare q matrix for output
qmat3 <- as.data.frame(qmatrix(tess3.obj, K = 3))
rownames(qmat3) <- rownames(dasgen)

#FF0099 = pink
#F0E442 = yellow
#E69F00 = orange
#56B4E9 = blue
#009E73 = green

## Use CreatePalette() to define color palettes.
my.colors <- c("#009E73","#56B4E9","#E69F00","#FF0099","#F0E442", "tan1", "red")
my.palette <- CreatePalette(my.colors, 10)

# Make high resolution map
k3 <- ggtess3Q(qmat3, latlongs, map.polygon=histdist, col.palette=my.palette,
               interpolation.model = FieldsKrigModel(10),
               resolution = c(1200,1200)) +
  geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
  xlim(113, 147) + 
  ylim(-22.5,-10) + 
  coord_equal() + 
  geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
  xlab("Longitude") +
  ylab("Latitude") + 
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        # axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text.x = element_blank(),
        # axis.text.x = element_text(size = 14, color="black"),
        axis.text.y = element_text(size = 14, color="black")) +
  # ggsn::scalebar(location="topleft",y.min=-22.5, y.max=-11, x.min=113, x.max=147, anchor=c(x=127.4,y=-22), 
  #        dist=500, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.09, st.size=4, height = 0.0365) +
  # north(location = "topright", scale = 0.2, symbol = 1,
  #       y.min=-22.5, y.max=-10, x.min=113, x.max=147, anchor = c(x=150,y=-10.2)) +
  annotate('text', 116.5, -11.25, label=paste("italic(k)==", 3, sep=""), parse=TRUE, hjust=1, size=8) +
  coord_quickmap()




## K = 4
# Prepare q matrix for output
qmat4 <- as.data.frame(qmatrix(tess3.obj, K = 4))
rownames(qmat4) <- rownames(dasgen)

## Use CreatePalette() to define color palettes.
my.colors <- c("#56B4E9","#009E73","#E69F00","#F0E442","#FF0099", "tan1", "red")
my.palette <- CreatePalette(my.colors, 10)

# Make high resolution map
k4 <- ggtess3Q(qmat4, latlongs, map.polygon=histdist, col.palette=my.palette,
               interpolation.model = FieldsKrigModel(10),
               resolution = c(1200,1200)) +
  geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
  xlim(113, 147) + 
  ylim(-22.5,-10) + 
  coord_equal() + 
  geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
  xlab("Longitude") +
  ylab("Latitude") + 
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_blank(),
        # axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text.x = element_blank(),
        # axis.text.x = element_text(size = 14, color="black"),
        axis.text.y = element_text(size = 14, color="black")) +
  # ggsn::scalebar(location="topleft",y.min=-22.5, y.max=-11, x.min=113, x.max=147, anchor=c(x=127.4,y=-22), 
  #        dist=500, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.09, st.size=4, height = 0.0365) +
  # north(location = "topright", scale = 0.2, symbol = 1,
  #       y.min=-22.5, y.max=-10, x.min=113, x.max=147, anchor = c(x=150,y=-10.2)) +
  annotate('text', 116.5, -11.25, label=paste("italic(k)==", 4, sep=""), parse=TRUE, hjust=1, size=8) +
  coord_quickmap()




## K = 5
# Prepare q matrix for output
qmat5 <- as.data.frame(qmatrix(tess3.obj, K = 5))
rownames(qmat5) <- rownames(dasgen)

#FF0099 = pink
#F0E442 = yellow
#E69F00 = orange
#56B4E9 = blue
#009E73 = green

## Use CreatePalette() to define color palettes.
my.colors <- c("#E69F00","#56B4E9","#009E73","#FF0099","#F0E442", "tan1", "red")
my.palette <- CreatePalette(my.colors, 10)

# Make high resolution map
k5 <- ggtess3Q(qmat5, latlongs, map.polygon=histdist, col.palette=my.palette,
               interpolation.model = FieldsKrigModel(10),
               resolution = c(1200,1200)) +
  geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
  xlim(113, 147) + 
  ylim(-22.5,-10) + 
  coord_equal() + 
  geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
  xlab("Longitude") +
  ylab("Latitude") + 
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        # axis.title.x=element_blank(),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        # axis.text.x = element_blank(),
        axis.text.x = element_text(size = 14, color="black"),
        axis.text.y = element_text(size = 14, color="black")) +
  ggsn::scalebar(location="topleft",y.min=-22.5, y.max=-11, x.min=113, x.max=147, anchor=c(x=127.4,y=-22),
         dist=500, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.10, st.size=5, height = 0.0365) +
  # north(location = "topright", scale = 0.2, symbol = 1,
  #       y.min=-22.5, y.max=-10, x.min=113, x.max=147, anchor = c(x=150,y=-10.2)) +
  annotate('text', 116.5, -11.25, label=paste("italic(k)==", 5, sep=""), parse=TRUE, hjust=1, size=8) +
  coord_quickmap()



## Stack and output all maps
# Organise layout of plots
lay <- rbind(c(1,1,1),
             c(1,1,1),
             c(1,1,1),
             c(2,2,2),
             c(2,2,2),
             c(2,2,2),
             c(3,3,3),
             c(3,3,3),
             c(3,3,3),
             c(4,4,4),
             c(4,4,4),
             c(4,4,4),
             c(4,4,4))

library(gridExtra)
pdffilename <- paste("output/tess3r_map_plots_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", height=13, width=8)
grid.arrange(k2,k3,k4,k5, layout_matrix=lay)
dev.off()






##################################################
## Genome scan for signs of selection ##
##################################################

# This is using tess3r to do a genome scan for signs of selection
# See Caye et al. 2016 and 2018 for details of algorithms

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
popns <- samps$popn
samps$region[samps$popn=="Astell Island"] <- "English Company Islands"

# Impute missing values in coordinates
samps[5:6] <- sapply(samps[5:6], function(x){
  ave(x, samps[[4]], FUN = function(.x) {
    .x[is.na(.x)] <- mean(.x, na.rm = TRUE)
    .x
  })
})



# tess3r package newer and better than LEA
#devtools::install_github("bcm-uga/TESS3_encho_sen")
library("tess3r")

## Prepare data
dasgen[1:10,1:10]
Y <- dasgen

## Create tess3r project
# Need lat and longs
latlongs <- as.matrix(samps[,5:6])
latlongs <- latlongs[,c(2,1)]

# set up allinfo for later
yeahcats <- aggregate(samps[ ,5:6], list(samps$popn), mean)


# Runs (latent factors = ancestral populations??) K = 1:x with cross-entropy calcs, each done with y
# repetitions.
tess3.obj <- tess3(X = Y, coord = latlongs, K = 1:20, rep=10, # including rep just puts error bars onto scree plot
                   method = "projected.ls", ploidy = 2, openMP.core.num = 7)

# plot cross-entropy criterion of all runs of the project
plot(tess3.obj, pch = 19, col = "blue",
     xlab = "Number of ancestral populations",
     ylab = "Cross-validation score")


library(dplyr)
library(reshape)
library(ggplot2)



# retrieve tess3 results for K = x
p.values <- pvalue(tess3.obj, K = 4) # chose the number of k we wish to test
hist(p.values, col = "lightblue", breaks=15)

# Benjamini-Hochberg algorithm
L = length(p.values)
fdr.level = 1e-4 # This is the false discovery rate (1e-2 = 0.01 = 1 SNP in 100)
w = which(sort(p.values) < fdr.level * (1:L)/L)
candidates = order(p.values)[w]
length(candidates) # This is the number of candidate loci for selection


# Output manhattan plot
# I have adjusted the y axis and removed title to make better for publication
pdffilename <- paste("plots/Rangewide_Tess3R_manhattanplot", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=7, height=5)
plot(p.values, main = NULL,
     pch=19,
     xlab = "Locus",
     ylab = "-log10(p-value)",
     cex = .3, col = "grey",
     yaxp = c(0,20,5))

points(candidates, -log10(p.values)[candidates],
       pch = 19, cex = .8, col = "black")
dev.off()













#######################################
#             Tajima's D              #
#######################################

# Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

library(dartR)
# Make function to calculate D (function provided by Renee Catullo)
get_tajima_D <- function(x){
  require(dartR) # possibly not needed for a function in an R package?
  
  # Find allele frequencies (p1 and p2) for every locus in every population
  allele_freqs <- gl.percent.freq(x)
  names(allele_freqs)[names(allele_freqs) == "frequency"] <- "p1"
  allele_freqs$p1 <- allele_freqs$p1 / 100
  allele_freqs$p2 <- 1 - allele_freqs$p1
  
  # Get the names of all the populations
  pops <- unique(allele_freqs$popn)
  
  #split each population
  allele_freqs_by_pop <- split(allele_freqs, allele_freqs$popn)
  
  # Internal function to calculate pi
  calc_pi <- function(allele_freqs) {
    n = allele_freqs$nobs * 2  # vector of n values
    pi_sqr <- allele_freqs$p1 ^ 2 + allele_freqs$p2 ^ 2
    h = (n / (n - 1)) * (1 - pi_sqr) # vector of values of h
    sum(h) # return pi, which is the sum of h across loci
  }
  
  get_tajima_D_for_one_pop <- function(allele_freqs_by_pop) {
    pi <- calc_pi(allele_freqs_by_pop)
    
    #Calculate number of segregating sites, ignoring missing data (missing data will not appear in teh allele freq calcualtions)
    #S <- sum(!(allele_freqs_by_pop$p1 == 0 | allele_freqs_by_pop$p1 == 1))
    S <- sum(allele_freqs_by_pop$p1 >0 & allele_freqs_by_pop$p1 <1)
    if(S == 0) {
      warning("No segregating sites")
      data.frame(pi = NaN, 
                 S = NaN, 
                 D = NaN, 
                 Pval.normal = NaN, 
                 Pval.beta = NaN)
    }
    
    n <- mean(allele_freqs_by_pop$nobs * 2 )
    
    tmp <- 1:(n - 1)
    a1 <- sum(1/tmp)
    a2 <- sum(1/tmp^2)
    b1 <- (n + 1)/(3 * (n - 1))
    b2 <- 2 * (n^2 + n + 3)/(9 * n * (n - 1))
    c1 <- b1 - 1/a1
    c2 <- b2 - (n + 2)/(a1 * n) + a2/a1^2
    e1 <- c1/a1
    e2 <- c2/(a1^2 + a2)
    
    #calculate D and do beta testing
    D <- (pi - S/a1) / sqrt(e1 * S + e2 * S * (S - 1))
    Dmin <- (2/n - 1/a1)/sqrt(e2)
    Dmax <- ((n/(2*(n - 1))) - 1/a1)/sqrt(e2)
    tmp1 <- 1 + Dmin * Dmax
    tmp2 <- Dmax - Dmin
    a <- -tmp1 * Dmax/tmp2
    b <- tmp1 * Dmin/tmp2
    p <- pbeta((D - Dmin)/tmp2, b, a)
    p <- ifelse(p < 0.5, 2 * p, 2 * (1 - p))
    
    data.frame(pi = pi, 
               S = S, 
               D = D, 
               Pval.normal = 2 * pnorm(-abs(D)), 
               Pval.beta = p)
  }
  
  output <- do.call("rbind", lapply(allele_freqs_by_pop, 
                                    get_tajima_D_for_one_pop))
  data.frame(population = rownames(output), output, row.names = NULL)
}


## Import data into genlight object
# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn

geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Remove the small (< 6 individuals) populations
smallpops <- as.data.frame(table(popns))[as.data.frame(table(popns))$Freq<6,]
smallpops <- as.character(smallpops$popns)
for(i in smallpops){
  genotypes <- genotypes[genotypes@pop!=i,]
}

## Convert to genlight object
genotypes <- gi2gl(genotypes, parallel = TRUE)

## Calculate Tajima's D for each population
tajD_MAC3 <- get_tajima_D(genotypes)
tajD_MAC3

# Round to 2 dps
round_df <- function(x, digits) {
  numeric_columns <- sapply(x, mode) == 'numeric'
  x[numeric_columns] <-  round(x[numeric_columns], digits)
  x
}
tajD_MAC3 <- round_df(tajD_MAC3, 2)

## Output results
write.csv(tajD_MAC3,"output/tajimaD_MAC_3.csv",row.names = FALSE)













###################################
## Fix up some stupid coordinates #
###################################

library(sf)
library(dplyr)

# Read in file ("easting" and "northing" are the header names used further down)
df <- read.csv("output/coords_for_fixing160920.csv",header=TRUE)

# Separate the zones into unique dataframes
df53 <- df[df$zone==53,]
df52 <- df[df$zone==52,]

# For each zone, convert to geometry object
df_sf1 <- st_as_sf(df52,
                  coords = c("easting", "northing"),
                  remove = FALSE,
                  crs = "+proj=utm +zone=52 +datum=WGS84 +south")

df_sf2 <- st_as_sf(df53,
                  coords = c("easting", "northing"),
                  remove = FALSE,
                  crs = "+proj=utm +zone=53 +datum=WGS84 +south")


# For each zone, transform into new projection thingy
df_sf_lonlat1 <- st_transform(df_sf1, crs = "+proj=longlat +datum=WGS84") %>% 
  mutate(long_degrees = st_coordinates(.)[,1],
         lat_degrees = st_coordinates(.)[,2])

df_sf_lonlat2 <- st_transform(df_sf2, crs = "+proj=longlat +datum=WGS84") %>% 
  mutate(long_degrees = st_coordinates(.)[,1],
         lat_degrees = st_coordinates(.)[,2])



## Check that decimal degrees are mapping correctly
library(ggplot2)
library(ggsn)
# Get basemap
w2hr <- map_data("world2Hires", xlim = c(130,137), ylim = c(-14.5,-10), lforce="e") # Featureless outline - check extent of map is suitable for your sites

# Make map that shows points and check if they look correct
ggplot() +
  geom_polygon(data = w2hr, aes(x=long, y = lat, group = group), fill = "grey90", color = "grey30", size=0.1) + 
  coord_fixed(1) +
  coord_equal(expand=FALSE) +
  geom_point(data=as.data.frame(df_sf_lonlat1),aes(x=long_degrees, y=lat_degrees)) +
  geom_point(data=as.data.frame(df_sf_lonlat2),aes(x=long_degrees, y=lat_degrees))


## Combine zones and output new data
df_dd <- rbind(as.data.frame(df_sf_lonlat1),as.data.frame(df_sf_lonlat2))
df_dd$geometry <- NULL

# write.csv(df_dd, "output/samps_decimal_degrees_160920.csv",row.names=FALSE)








##################################
# Check basic sfs for a few pops #
##################################

rm(list = ls())
currentDate <- Sys.Date()

dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

samps <- read.csv(file="output/sampinfo250221_sex.csv",header=TRUE,row.names=NULL,na.strings = c("NA",""))
samps <- subset(samps,samps$sample %in% rownames(dasgen))

artrange <- subset(dasgen, rownames(dasgen) %in% samps$sample[samps$popn=="Artesian Range"])
outdar <- subset(dasgen, rownames(dasgen) %in% samps$sample[samps$popn=="Outer Darwin"])
weipa <- subset(dasgen, rownames(dasgen) %in% samps$sample[samps$popn=="Weipa"])
mareeba <- subset(dasgen, rownames(dasgen) %in% samps$sample[samps$popn=="Mareeba"])

mac1 <- colSums(artrange,na.rm = TRUE)
hist(mac1,breaks=50)
mac2 <- colSums(outdar,na.rm = TRUE)
hist(mac2,breaks=50)
mac3 <- colSums(weipa,na.rm = TRUE)
hist(mac3,breaks=25)
mac4 <- colSums(mareeba,na.rm = TRUE)
hist(mac4,breaks=25)








####################################################################
### Get lats,longs,samp sizes,treatments for all populations sampled

# Clear environment and set up working dir and datestamp
rm(list = ls())
setwd("E:/nor_quoll_genetics")
currentDate <- Sys.Date() # Add current date to environment for output of file names with date

# Read in data and check
library(readxl)
df <- read_excel("SuppMat_sample_metadata.xlsx")

df[13:14] <- sapply(df[13:14], function(x){
  ave(x, df[[4]], FUN = function(.x) {
    .x[is.na(.x)] <- mean(.x, na.rm = TRUE)
    .x
  })
})

# Get population names sorted
library(lubridate)
df$collection_date <- format(as.Date(df$collection_date, format="%Y-%m-%d"),"%Y-%m-%d")
df$year <- as.numeric(year(df$collection_date))
df$population[df$population=="Astell Island" & df$year<2010] <- "Astell Island 2005"
df$population[df$population=="Astell Island" & df$collection_date>2010] <- "Astell Island 2016"
df$population[df$population=="Kapalga" & df$collection_date<2010] <- "Kapalga 1994"
df$population[df$population=="Kapalga" & df$collection_date>2010] <- "Kapalga 2014"
df$population[df$population=="East Alligator" & df$collection_date<2010] <- "East Alligator 2003"
df$population[df$population=="East Alligator" & df$collection_date>2015] <- "East Alligator 2017"
df$population[df$population=="East Alligator" & df$collection_date<2015] <- "East Alligator 2014"
df$population[df$population=="Darwin" & df$collection_date>2017] <- "Darwin 2019"
df$population[df$population=="Darwin" & df$collection_date<2017] <- "Darwin 2000"
df$population[df$population=="Darwin" & is.na(df$collection_date)] <- "Darwin 2000"

# Obtain mean values for lat and long
meanlatlong <- aggregate(df[, 13:14], list(df$population), mean)

# Count number of rows/samples per population
library(plyr)
numsamps <- count(df$population)
colnames(numsamps) <- c("Group.1","freq")

# Combine dfs
sum.sites <- merge(numsamps,meanlatlong,by="Group.1")
colnames(sum.sites) <- c("population","sampsize","latitude","longitude")

write.csv(sum.sites,"output/sampled_sites_summary.csv",row.names = FALSE)




nrow(sum.sites[sum.sites$sampsize>6,])















####################################
# Get sequences for BEAST analysis #
####################################

# We need ddRAD loci (reads) with > 1 SNP
# Get linked SNPs and then extract the reads with those SNPs
# from the BAM files for the samples/populations of interest


## Get samples of interest

# Read in data and check
# Run code from lines 184 to 378 (to get the SNPRelate list of SNPs that are not in LD;
# change ld.threshold=0.01 and sliding window to 500 or thereabouts)
# Find the difference between the full list of filtered SNPs and the LD filtered list to get the linked SNPs
check <- rownames(dasgen)
linkgen <- subset(dasgen, !(rownames(dasgen) %in% LDFilt)) # Dataset of SNPs that are in close proximity with others
linkgen <- rownames(linkgen) # SNP names in close proximity

# Randomly select x ddRAD loci to use from list of SNPs in close proximity to other SNPs
set.seed(33)
snpfindseq <- sample(linkgen, 2500, replace=FALSE)
snpfindseq <- as.data.frame(snpfindseq)
library(stringr)
snpfindseq <- as.data.frame(str_split_fixed(snpfindseq$snpfindseq, "-", 2))
colnames(snpfindseq) <- c("chromnum","position")
snpfindseq$chromnum <- as.numeric(snpfindseq$chromnum)
snpfindseq$position <- as.numeric(snpfindseq$position)

# Get samples of interest
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% colnames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"

samps <- subset(samps, samps$popn %in% c("Weipa","Mareeba",
                                "Groote West","East Alligator 2017","East Alligator 2003","Astell Island 2016",
                                "Bachsten Creek","King Leopold West","Koolan Island","Millstream Chichester"))
sampsofinterest <- samps$sample
sampsofinterest <- sampsofinterest[!grepl("x",sampsofinterest)]


### Extract reads from selected sample BAM files at selected SNPs

# Load packages
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install("Rsamtools")
library(Rsamtools)
library("quantmod")


## Need to run a for loop through samples to pull the relevant reads out of the BAM files

# Set regions and parameters to obtain
library(dplyr)
snpfindseq <- snpfindseq %>% arrange(chromnum,position)
snpfindseq$start <- snpfindseq$position-1
snpfindseq$end <- snpfindseq$position+1
unique(snpfindseq$chromnum)
length(unique(snpfindseq$chromnum))

# This is basically a list of chromosomes/scaffolds
which <- IRangesList("36"=IRanges(snpfindseq$start[snpfindseq$chromnum=="36"], snpfindseq$end[snpfindseq$chromnum=="36"]),
                     "53"=IRanges(snpfindseq$start[snpfindseq$chromnum=="53"], snpfindseq$end[snpfindseq$chromnum=="53"]),
                     "67"=IRanges(snpfindseq$start[snpfindseq$chromnum=="67"], snpfindseq$end[snpfindseq$chromnum=="67"]),
                     "68"=IRanges(snpfindseq$start[snpfindseq$chromnum=="68"], snpfindseq$end[snpfindseq$chromnum=="68"]),
                     "69"=IRanges(snpfindseq$start[snpfindseq$chromnum=="69"], snpfindseq$end[snpfindseq$chromnum=="69"]),
                     "70"=IRanges(snpfindseq$start[snpfindseq$chromnum=="70"], snpfindseq$end[snpfindseq$chromnum=="70"]),
                     "71"=IRanges(snpfindseq$start[snpfindseq$chromnum=="71"], snpfindseq$end[snpfindseq$chromnum=="71"]),
                     "72"=IRanges(snpfindseq$start[snpfindseq$chromnum=="72"], snpfindseq$end[snpfindseq$chromnum=="72"]),
                     "74"=IRanges(snpfindseq$start[snpfindseq$chromnum=="74"], snpfindseq$end[snpfindseq$chromnum=="74"]),
                     "75"=IRanges(snpfindseq$start[snpfindseq$chromnum=="75"], snpfindseq$end[snpfindseq$chromnum=="75"]),
                     "76"=IRanges(snpfindseq$start[snpfindseq$chromnum=="76"], snpfindseq$end[snpfindseq$chromnum=="76"]),
                     "89"=IRanges(snpfindseq$start[snpfindseq$chromnum=="89"], snpfindseq$end[snpfindseq$chromnum=="89"]),
                     "90"=IRanges(snpfindseq$start[snpfindseq$chromnum=="90"], snpfindseq$end[snpfindseq$chromnum=="90"]),
                     "91"=IRanges(snpfindseq$start[snpfindseq$chromnum=="91"], snpfindseq$end[snpfindseq$chromnum=="91"]),
                     "94"=IRanges(snpfindseq$start[snpfindseq$chromnum=="94"], snpfindseq$end[snpfindseq$chromnum=="94"]),
                     "95"=IRanges(snpfindseq$start[snpfindseq$chromnum=="95"], snpfindseq$end[snpfindseq$chromnum=="95"]),
                     "96"=IRanges(snpfindseq$start[snpfindseq$chromnum=="96"], snpfindseq$end[snpfindseq$chromnum=="96"]),
                     "97"=IRanges(snpfindseq$start[snpfindseq$chromnum=="97"], snpfindseq$end[snpfindseq$chromnum=="97"]),
                     "98"=IRanges(snpfindseq$start[snpfindseq$chromnum=="98"], snpfindseq$end[snpfindseq$chromnum=="98"]),
                     "99"=IRanges(snpfindseq$start[snpfindseq$chromnum=="99"], snpfindseq$end[snpfindseq$chromnum=="99"]),
                     "100"=IRanges(snpfindseq$start[snpfindseq$chromnum=="100"], snpfindseq$end[snpfindseq$chromnum=="100"]),
                     "101"=IRanges(snpfindseq$start[snpfindseq$chromnum=="101"], snpfindseq$end[snpfindseq$chromnum=="101"]),
                     "124"=IRanges(snpfindseq$start[snpfindseq$chromnum=="124"], snpfindseq$end[snpfindseq$chromnum=="124"]),
                     "125"=IRanges(snpfindseq$start[snpfindseq$chromnum=="125"], snpfindseq$end[snpfindseq$chromnum=="125"]),
                     "126"=IRanges(snpfindseq$start[snpfindseq$chromnum=="126"], snpfindseq$end[snpfindseq$chromnum=="126"]),
                     "128"=IRanges(snpfindseq$start[snpfindseq$chromnum=="128"], snpfindseq$end[snpfindseq$chromnum=="128"]),
                     "129"=IRanges(snpfindseq$start[snpfindseq$chromnum=="129"], snpfindseq$end[snpfindseq$chromnum=="129"]),
                     "130"=IRanges(snpfindseq$start[snpfindseq$chromnum=="130"], snpfindseq$end[snpfindseq$chromnum=="130"]),
                     "132"=IRanges(snpfindseq$start[snpfindseq$chromnum=="132"], snpfindseq$end[snpfindseq$chromnum=="132"]),
                     "133"=IRanges(snpfindseq$start[snpfindseq$chromnum=="133"], snpfindseq$end[snpfindseq$chromnum=="133"]),
                     "135"=IRanges(snpfindseq$start[snpfindseq$chromnum=="135"], snpfindseq$end[snpfindseq$chromnum=="135"]),
                     "136"=IRanges(snpfindseq$start[snpfindseq$chromnum=="136"], snpfindseq$end[snpfindseq$chromnum=="136"]),
                     "137"=IRanges(snpfindseq$start[snpfindseq$chromnum=="137"], snpfindseq$end[snpfindseq$chromnum=="137"]),
                     "138"=IRanges(snpfindseq$start[snpfindseq$chromnum=="138"], snpfindseq$end[snpfindseq$chromnum=="138"]),
                     "139"=IRanges(snpfindseq$start[snpfindseq$chromnum=="139"], snpfindseq$end[snpfindseq$chromnum=="139"]),
                     "140"=IRanges(snpfindseq$start[snpfindseq$chromnum=="140"], snpfindseq$end[snpfindseq$chromnum=="140"]),
                     "141"=IRanges(snpfindseq$start[snpfindseq$chromnum=="141"], snpfindseq$end[snpfindseq$chromnum=="141"]),
                     "142"=IRanges(snpfindseq$start[snpfindseq$chromnum=="142"], snpfindseq$end[snpfindseq$chromnum=="142"]),
                     "144"=IRanges(snpfindseq$start[snpfindseq$chromnum=="144"], snpfindseq$end[snpfindseq$chromnum=="144"]),
                     "147"=IRanges(snpfindseq$start[snpfindseq$chromnum=="147"], snpfindseq$end[snpfindseq$chromnum=="147"]),
                     "148"=IRanges(snpfindseq$start[snpfindseq$chromnum=="148"], snpfindseq$end[snpfindseq$chromnum=="148"]),
                     "151"=IRanges(snpfindseq$start[snpfindseq$chromnum=="151"], snpfindseq$end[snpfindseq$chromnum=="151"]),
                     "155"=IRanges(snpfindseq$start[snpfindseq$chromnum=="155"], snpfindseq$end[snpfindseq$chromnum=="155"]),
                     "166"=IRanges(snpfindseq$start[snpfindseq$chromnum=="166"], snpfindseq$end[snpfindseq$chromnum=="166"]),
                     "171"=IRanges(snpfindseq$start[snpfindseq$chromnum=="171"], snpfindseq$end[snpfindseq$chromnum=="171"]),
                     "177"=IRanges(snpfindseq$start[snpfindseq$chromnum=="177"], snpfindseq$end[snpfindseq$chromnum=="177"]),
                     "183"=IRanges(snpfindseq$start[snpfindseq$chromnum=="183"], snpfindseq$end[snpfindseq$chromnum=="183"]),
                     "185"=IRanges(snpfindseq$start[snpfindseq$chromnum=="185"], snpfindseq$end[snpfindseq$chromnum=="185"]),
                     "197"=IRanges(snpfindseq$start[snpfindseq$chromnum=="197"], snpfindseq$end[snpfindseq$chromnum=="197"]),
                     "212"=IRanges(snpfindseq$start[snpfindseq$chromnum=="212"], snpfindseq$end[snpfindseq$chromnum=="212"]),
                     "221"=IRanges(snpfindseq$start[snpfindseq$chromnum=="221"], snpfindseq$end[snpfindseq$chromnum=="221"]),
                     "222"=IRanges(snpfindseq$start[snpfindseq$chromnum=="222"], snpfindseq$end[snpfindseq$chromnum=="222"]),
                     "226"=IRanges(snpfindseq$start[snpfindseq$chromnum=="226"], snpfindseq$end[snpfindseq$chromnum=="226"]),
                     "244"=IRanges(snpfindseq$start[snpfindseq$chromnum=="244"], snpfindseq$end[snpfindseq$chromnum=="244"]),
                     "259"=IRanges(snpfindseq$start[snpfindseq$chromnum=="259"], snpfindseq$end[snpfindseq$chromnum=="259"]),
                     "260"=IRanges(snpfindseq$start[snpfindseq$chromnum=="260"], snpfindseq$end[snpfindseq$chromnum=="260"]),
                     "263"=IRanges(snpfindseq$start[snpfindseq$chromnum=="263"], snpfindseq$end[snpfindseq$chromnum=="263"]),
                     "266"=IRanges(snpfindseq$start[snpfindseq$chromnum=="266"], snpfindseq$end[snpfindseq$chromnum=="266"]),
                     "269"=IRanges(snpfindseq$start[snpfindseq$chromnum=="269"], snpfindseq$end[snpfindseq$chromnum=="269"]),
                     "306"=IRanges(snpfindseq$start[snpfindseq$chromnum=="306"], snpfindseq$end[snpfindseq$chromnum=="306"]),
                     "309"=IRanges(snpfindseq$start[snpfindseq$chromnum=="309"], snpfindseq$end[snpfindseq$chromnum=="309"]),
                     "310"=IRanges(snpfindseq$start[snpfindseq$chromnum=="310"], snpfindseq$end[snpfindseq$chromnum=="310"]),
                     "320"=IRanges(snpfindseq$start[snpfindseq$chromnum=="320"], snpfindseq$end[snpfindseq$chromnum=="320"]),
                     "349"=IRanges(snpfindseq$start[snpfindseq$chromnum=="349"], snpfindseq$end[snpfindseq$chromnum=="349"]),
                     "353"=IRanges(snpfindseq$start[snpfindseq$chromnum=="353"], snpfindseq$end[snpfindseq$chromnum=="353"]),
                     "386"=IRanges(snpfindseq$start[snpfindseq$chromnum=="386"], snpfindseq$end[snpfindseq$chromnum=="386"]),
                     "397"=IRanges(snpfindseq$start[snpfindseq$chromnum=="397"], snpfindseq$end[snpfindseq$chromnum=="397"]))

what <- c("rname", "strand", "pos", "qwidth", "seq")
param <- ScanBamParam(which=which, what=what)

# quickBamFlagSummary("E:/nor_quoll_genetics/sequence_data/files_from_cloud/bamfiles/DHAL0002.bam")



## Scan the bam files of interest and extract the selected sequences

# Make function for calling in loop
.unlist <- function (x){
  ## do.call(c, ...) coerces factor to integer, which is undesired
  x1 <- x[[1L]]
  if (is.factor(x1)) {
    structure(unlist(x), class = "factor", levels = levels(x1))
  } else {
    do.call(c, x)
  }
}

# Make list to fill with data
all_samps_seqs <- list()

# Run through all samples of interest
for(i in sampsofinterest){

    # indexBam("~Powel-ER1133_sorted.bam") # Index files already provided in folder
  bam <- scanBam(paste("E:/nor_quoll_genetics/sequence_data/files_from_cloud/bamfiles/", i, ".bam", sep=""), param=param)
  
  bam <- unname(bam) # names not useful in unlisted result
  elts <- setNames(bamWhat(param), bamWhat(param))
  lst <- lapply(elts, function(elt) .unlist(lapply(bam, "[[", elt)))
  head(do.call("DataFrame", lst))
  
  library(dplyr)
  lst <- as.data.frame(lst)
  lst$snp <- paste(lst$rname,lst$pos,sep="_")
  lst <- lst %>% group_by(snp) %>% count(seq) %>% top_n(1) # includes ties
  lst <- lst[lst$n > 10 , ] # Removes some weird sequences that overlap with other sequences
  lst <- lst[!duplicated(lst$snp), ] # Remove ties
  
  chrpos <- as.data.frame(str_split_fixed(lst$snp, "_", 2))
  colnames(chrpos) <- c("chromnum","position")
  seqs <- cbind(chrpos,lst[,c(1:2)])
  
  seqs$sample <- i  # maybe you want to keep track of which iteration produced it?
  all_samps_seqs[[i]] <- seqs # add it to your list
  
}

# We now have a list of data frames with variable final number of SNPs
# Need to combine into single file of common SNP names

## rbind all dfs together, group by snp id and then remove groups with < x snps
# so that all samples are sequenced at the same snps and the same number of snps
library(dplyr)
library(data.table)
bind_samps_seqs <- rbindlist(all_samps_seqs)

check <- bind_samps_seqs %>% 
  group_by(sample) %>% 
  count()

hist(check$n, breaks=15)

bind_samps_seqs <- bind_samps_seqs %>% 
  group_by(sample) %>% 
  filter(n() >= 800) # Prune some crap samples out based on number of SNPs


## Get metadata for all retained samples
samps <- subset(samps,samps$sample %in% unique(bind_samps_seqs$sample))

# Add populations
samppop <- samps[ , c(1,4)]
bind_samps_seqs <- left_join(bind_samps_seqs,samppop,by="sample")



## Filter to pop of interest
# Just done one by one for each population
library(dplyr)
weipasnps <- bind_samps_seqs %>% 
  group_by(sample) %>% 
  filter(popn == "Millstream Chichester") # Can change the population name but leave everything else

check <- weipasnps %>% 
  group_by(snp) %>% 
  count()
hist(check$n)

weipasnps <- weipasnps %>% 
  group_by(snp) %>% 
  filter(n() >= length(unique(weipasnps$sample))) # Prune SNPs to only those in all samples


## Select a few loci (reads) to convert to nexus files for BEAST
# Check some combinations of individuals to make sure there's a couple of variable sites in the pop
library(Biostrings)

for(i in c(201:400)){ # I just run through snps 100 at a time and eyeball results for variable loci
  
  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0240"][i])) # Mareeba
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0243"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0247"][i]))
  # 
  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0249"][i])) # Weipa
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0256"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0258"][i]))

  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0032"][i])) # Groote
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0071"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0137"][i]))
  # 
  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0145"][i])) # Astell Island 2016
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0151"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0169"][i]))
  # 
  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0508"][i])) # East Alligator 2003
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0513"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0519"][i]))

  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0177"][i])) # East Alligator 2017
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0180"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0204"][i]))
  # 
  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0350"][i])) # Bachsten
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0353"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0356"][i]))
  # 
  # s1 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0452"][i])) # Millstream Chichester
  # s2 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0455"][i]))
  # s3 <- as.integer(DNAString(weipasnps$seq[weipasnps$sample=="DHAL0460"][i]))
  
  print(c("SNP NUMBER",i))
  print(which(s1!=s2))
  print(which(s2!=s3))
  print(which(s1!=s3))
    
}


# Need to manually add lines to this vector based on above results for each population
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0452"][2],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][16],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][36],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][246],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][275],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][375],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][110],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][124],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][157],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][201]

))

# Make data frame to fill
weipa_dat <- as.data.frame(expand.grid(unique(weipasnps$sample),snpsofinterest))
colnames(weipa_dat) <- c("sample","snp")

weipa_dat <- left_join(weipa_dat,weipasnps,by=c("sample","snp"))


# output dat to tab-delimited text file to manually make nexus files for BEAST # CHANGE FILENAME
weipa_dat <- weipa_dat[ , c(1,5,2)]
write.table(weipa_dat, file="output/Millstream_variable_loci.txt", quote = FALSE, sep="\t", row.names = FALSE)







### This is the final set of SNPs retained for each population to have a skyline plot made in BEAST

## Bachsten SNPs to retain
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0350"][104],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][128],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][190],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][249],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][370],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][569],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][526],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][598],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][502],
  weipasnps$snp[weipasnps$sample=="DHAL0350"][294]
  ))


## Weipa SNPs to retain
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0249"][73],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][121],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][167],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][292],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][377],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][429],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][583],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][597],
  weipasnps$snp[weipasnps$sample=="DHAL0249"][615]
  ))

# Mareeba SNPs to retain
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0240"][17],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][23],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][28],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][31],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][75],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][88],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][138],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][189],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][296],
  weipasnps$snp[weipasnps$sample=="DHAL0240"][342]
  ))

## East Alligator 2003 SNPs of interest
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0508"][521],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][19],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][413],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][41],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][363],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][128],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][152],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][188],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][235],
  weipasnps$snp[weipasnps$sample=="DHAL0508"][346]
  ))

## Groote West SNPs to retain
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0032"][13],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][34],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][38],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][64],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][109],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][143],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][197],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][225],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][767],
  weipasnps$snp[weipasnps$sample=="DHAL0032"][469]
  ))


## Millstream Chichester SNPs of interest
snpsofinterest <- as.vector(c(
  weipasnps$snp[weipasnps$sample=="DHAL0452"][2],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][16],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][36],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][246],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][275],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][375],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][110],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][124],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][157],
  weipasnps$snp[weipasnps$sample=="DHAL0452"][201]
  ))










#########################
# Plot the SNeP results #
#########################

## Read in results
# WA
bachsten <- read.table("output/snep/bachsten.NeAll", header=TRUE)
bachsten$GenAgo <- bachsten$GenAgo+4
bachsten$pop <- "Bachsten Creek"
artrange <- read.table("output/snep/artrange2013.NeAll", header=TRUE)
artrange$pop <- "Artesian Range"
artrange$GenAgo <- artrange$GenAgo+5
millstream <- read.table("output/snep/millstream2014.NeAll", header=TRUE)
millstream$pop <- "Millstream Chichester NP"
millstream$GenAgo <- millstream$GenAgo+4
koolan <- read.table("output/snep/koolan2006.NeAll", header=TRUE)
koolan$pop <- "Koolan Island"
koolan$GenAgo <- koolan$GenAgo+12
mornington <- read.table("output/snep/mornington2012.NeAll", header=TRUE)
mornington$pop <- "Mornington Sanctuary"
mornington$GenAgo <- mornington$GenAgo+6
wunaaminwest <- read.table("output/snep/wunaaminwest2016.NeAll", header=TRUE)
wunaaminwest$pop <- "Wunaamin Miliwundi West"
wunaaminwest$GenAgo <- wunaaminwest$GenAgo+2
marblebar <- read.table("output/snep/marblebar.NeAll", header=TRUE)
marblebar$pop <- "Marble Bar"
marblebar$GenAgo <- marblebar$GenAgo+6

# NT
eastallig2003 <- read.table("output/snep/eastalligator.NeAll", header=TRUE)
eastallig2003$GenAgo <- eastallig2003$GenAgo+15
eastallig2003$pop <- "East Alligator (2003)"
eastallig2017 <- read.table("output/snep/eastallig2017.NeAll", header=TRUE)
eastallig2017$GenAgo <- eastallig2017$GenAgo+1
eastallig2017$pop <- "East Alligator (2017)"
kapalga <- read.table("output/snep/kapalga94.NeAll", header=TRUE)
kapalga$GenAgo <- kapalga$GenAgo+24
kapalga$pop <- "Kapalga"
outdar <- read.table("output/snep/outdar2003.NeAll", header=TRUE)
outdar$GenAgo <- outdar$GenAgo+15
outdar$pop <- "Outer Darwin"
groote <- read.table("output/snep/grootew2018.NeAll", header=TRUE)
groote$GenAgo <- groote$GenAgo
groote$pop <- "Groote West"
# marchinbar <- read.table("output/snep/marchinbar06.NeAll", header=TRUE) # Can't use marchinbar due to lack of diversity
# astell <- read.table("output/snep/astell2016.NeAll", header=TRUE)
# eastalgone <- read.table("output/snep/Output_Ne_eastalligator", header=TRUE)


# Queensland
maree <- read.table("output/snep/mareeba.NeAll", header=TRUE)
maree$GenAgo <- maree$GenAgo+3
maree$pop <- "Mareeba"
hopevale <- read.table("output/snep/hopevale2016.NeAll", header=TRUE)
hopevale$GenAgo <- hopevale$GenAgo+2
hopevale$pop <- "Hope Vale"
blackmtn <- read.table("output/snep/blackmtn2016.NeAll", header=TRUE)
blackmtn$GenAgo <- blackmtn$GenAgo+2
blackmtn$pop <- "Black Mountain"
weipa <- read.table("output/snep/weipa.NeAll", header=TRUE)
weipa$GenAgo <- weipa$GenAgo
weipa$pop <- "Weipa"
bmhp <- read.table("output/snep/bmhp2016.NeAll", header=TRUE)
bmhp$GenAgo <- bmhp$GenAgo+2
bmhp$pop <- "Black Mtn / Hope Vale"


# Combine all for facet plotting
qldpops <- rbind(maree,hopevale,blackmtn,bmhp, weipa)
topend <- rbind(eastallig2003,eastallig2017,kapalga,outdar)
kimberley <- rbind(bachsten,artrange,koolan,mornington,wunaaminwest)
pilbara <- rbind(millstream, marblebar)
groote <- groote

# Add regional names
qldpops$Region <- "Queensland"
topend$Region <- "Top End"
kimberley$Region <- "Kimberley"
pilbara$Region <- "Pilbara"
groote$Region <- "Groote Eylandt"

# Combine all regions
allregions <- rbind(qldpops,topend,kimberley,pilbara,groote)

# Order levels for colours to match properly
allregions$Region <- factor(allregions$Region, levels=c("Queensland","Groote Eylandt","Top End","Kimberley","Pilbara"))

# Order levels for colours to match properly
allregions$pop <- factor(allregions$pop, levels=c("Mareeba","Hope Vale","Black Mountain","Black Mtn / Hope Vale",
                                                  "Weipa","Groote West","East Alligator (2003)","East Alligator (2017)",
                                                  "Kapalga","Outer Darwin","Bachsten Creek","Artesian Range","Koolan Island",
                                                  "Mornington Sanctuary","Wunaamin Miliwundi West","Millstream Chichester NP",
                                                  "Marble Bar"))

# Convert number of generations to the year of each estimate
allregions$Year <- allregions$GenAgo*2 # generation time in Pacifici et al. 2013
allregions$Year <- 2018-allregions$Year


# Plot
library(ggplot2)
library(viridis)
snep_plot <- ggplot(data=allregions, aes(Year,Ne, group=pop, colour=Region)) +
  geom_point(size=0.9) +
  geom_line() +
  facet_wrap(~ Region, nrow=5, ncol=1, scales="fixed") +
  theme_bw() +
  scale_alpha(guide = "none") +
  labs(colour = "Region") +
  scale_colour_manual(values=c("#009E73","#56B4E9","#F0E442","#E69F00","#FF0099"), guide="none") +
  xlab("Generations ago") +
  ylab(expression(paste("Effective population size (",italic(N)*{}[e],")"))) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black")) +
  scale_x_reverse(limits=c(2000,1788)) +
  scale_y_continuous(limits=c(0,3700))


# Output to PDF
library(gridExtra)
pdffilename <- paste("output/SneP_Ne_history_", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=6, height=6)
snep_plot
dev.off()




## Plot with log scale
snep_plot <- ggplot(data=allregions, aes(Year,Ne, group=pop, colour=Region)) +
  geom_vline(xintercept=1788, linetype=2, size=0.6, alpha=0.3) + # European colonisation in Australia
  geom_vline(xintercept=1945, linetype=3, size=0.6, alpha=0.3) +
  geom_point(size=0.9) +
  geom_line() +
  facet_wrap(~ pop, nrow=6, ncol=3, scales="fixed") +
  theme_bw() +
  scale_colour_manual(values=c("#009E73","#56B4E9","#F0E442","#E69F00","#FF0099"), guide="none") +
  scale_shape_manual(values = rep(c(15,17,19,21),5)) +
  xlab("Generations ago") +
  ylab(expression(paste("Effective population size (",italic(N)*{}[e],")"))) +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black"))  +
  scale_y_log10(limits=c(35,20000), expand=c(0,0), breaks=c(10,100,1000,10000),
                labels = trans_format("log10", math_format(10^.x))) +
  scale_x_reverse(limits=c(2000,1740))



# Output to PDF
library(gridExtra)
pdffilename <- paste("output/SneP_Ne_history_log_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=8, height=8)
snep_plot
dev.off()







###############################
# Map all sites for Supp Info #
###############################

# Read in data and check
df <- read.csv("samp_submission/sampinfo250221.csv",na.strings=c(""," ","NA"))
str(df)
colnames(df)
head(df)
df$lat <- as.numeric(df$lat)
df$long <- as.numeric(df$long)
df$date <- format(as.Date(df$date, format="%d/%m/%Y"),"%Y-%m-%d")
df$popn <- as.factor(df$popn)
min(df$date,na.rm = TRUE) # oldest sample
max(df$date,na.rm = TRUE) # youngest sample
str(df)



### Map all samples used

# Load mapping packages
if (!require("ggplot2")) install.packages("ggplot2")
if (!require("mapdata")) install.packages("mapdata")
if (!require("ggsn")) install.packages("ggsn")

## Set up limits of maps to save time
# Need to replace missing values of lat long with the popn mean
df[5:6] <- sapply(df[5:6], function(x){
  ave(x, df[[4]], FUN = function(.x) {
    .x[is.na(.x)] <- mean(.x, na.rm = TRUE)
    .x
  })
})

mapdf <- df[!is.na(df$lat),] # In case there's any missing vals left..
xmin <- floor(min(mapdf$long))-1
xmax <- ceiling(max(mapdf$long))+1
ymin <- floor(min(mapdf$lat))-1.5
ymax <- ceiling(max(mapdf$lat))+1

# Create basemap
w2hr <- map_data("world2Hires", xlim = c(xmin,xmax), ylim = c(ymin,ymax), lforce="e") # Featureless

# Obtain mean values for lat and long
meanlatlong <- aggregate(mapdf[, 5:6], list(mapdf$popn), mean)

# Count number of rows/samples per population
library(plyr)
numsamps <- count(mapdf$popn)
colnames(numsamps) <- c("Group.1","freq")

# Combine dfs
sum.sites <- merge(numsamps,meanlatlong,by="Group.1")
colnames(sum.sites) <- c("population","sampsize","latitude","longitude")

library(ggrepel)

# Adjust scalebar and north arrow scale as needed
quollmap2 <- ggplot() +
  geom_polygon(data = w2hr, aes(x=long, y = lat, group = group), fill = "grey85", color = "black") + 
  coord_fixed(1) +
  coord_equal(expand=FALSE) +
  theme_bw() +
  scale_alpha(guide = "none") +
  theme(panel.grid=element_blank()) +
  xlab("Longitude") +
  ylab("Latitude") +
  theme(axis.title.y=element_text(size=14, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=14, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 14, color="black")) +
  geom_point(data = sum.sites, aes(x = longitude, y = latitude), color = "black",
             size = sqrt(sum.sites$sampsize)*1.25, pch=21, fill="dodgerblue2") +
  # geom_text(data = sum.sites, aes(x = longitude, y = latitude, label=population)) +
  geom_text_repel(data = sum.sites, aes(x = longitude, y = latitude, label = population), force=25, max.overlaps=15,
                   box.padding=0.05, point.padding=1, segment.color='grey30', min.segment.length = 0.25) +
  north(location = "topleft", scale = 0.18, symbol = 1,
        y.min=ymin, y.max=ymax, x.min=xmin+15, x.max=xmax,
        anchor = c(x=xmin, y=ymax-(2/100*(xmax-xmin)))) +
  scale_y_continuous(breaks=c(-23, -19, -15, -11)) +
  scale_x_continuous(breaks=c(116,126,136,146)) +
  ggsn::scalebar(location="bottomleft",y.min=ymin, y.max=ymax, x.min=xmin, x.max=xmax, 
                 dist=500, dist_unit="km", transform=TRUE, model='WGS84',
                 anchor = c(x = xmax-11.5, y = ymin+0.8), st.color="black", st.dist=-0.05, st.size=4)

quollmap2


## Output plots of species by year
pdffilename <- paste("output/quolls_map",currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=11, height=8)
quollmap2
dev.off()


## Some minor edits in Inkscape to clean up labelling.








###############################
# Private Alleles calculation #
###############################


### First by region (QLD, NT, WA)

library(hierfstat)

# Clear environment
rm(list = ls())
currentDate <- Sys.Date() # Add current date to environment for output of file names with date


# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn
state <- samps$state


# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = state, ncode=1)  # Takes a while with large datasets
genotypes


## Bootstrap samples to get number of private alleles standardised by sample size

# Set up results dataframe to append to
privals <- as.data.frame(matrix(nrow=1,ncol=3))

# Add columns to data
geno.types$popns <- state
geno.types$sample <- rownames(geno.types)

# Run loop
library(poppr)
library(dplyr)
library(adegenet)

for(i in 1:100){
  
  # Randomly sample 25 individuals from each state
  df <- geno.types %>% group_by(popns) %>% sample_n(size = 25, replace=FALSE)
  df <- as.data.frame(df)
  rownames(df) <- df$sample
  df$sample <- NULL
  popns <- df$popns
  df$popns <- NULL
  
  ## Convert to genind object
  genotypes <- df2genind(df, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
  
  # Set strata
  strata(genotypes) <- data.frame(x = pop(genotypes))
  
  # Count private alleles with poppr
  priv <- private_alleles(genotypes, count.alleles=FALSE, level="population", report="table")
  priv <- rowSums(priv)
  
  # Add results to output
  colnames(privals) <- names(priv)
  privals[i,] <- priv
  
}


# Get mean values
privmeans <- as.data.frame(colMeans(privals))
privmeans

# Obtain standard errors for each popn
library("plotrix")
privmeans$se <- apply(privals,2,function(x){std.error(x)})
colnames(privmeans) <- c("mean","se")
privmeans <- round(privmeans,2)

# Output to csv
write.csv(privmeans,"output/PrivateAllelesRegion010921.csv",row.names=TRUE)


# # Check sig
# library(reshape2)
# dfprivs <- melt(privals)
# mod1 <- glm(value ~ variable, data=dfprivs)
# car::Anova(mod1,test.statistic="F")
# 
# # Check contrasts (Tukey multiple comparison)
# library(emmeans)
# marginal = lsmeans(mod1, ~ variable)
# pairs(marginal, adjust="tukey")
# # Done







### Now by island vs mainland

library(hierfstat)

# Clear environment
rm(list = ls())
currentDate <- Sys.Date() # Add current date to environment for output of file names with date


# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn
state <- samps$state
samps$treatment[samps$treatment=="post-toad"] <- "mainland"
samps$treatment[samps$treatment=="pre-toad"] <- "mainland"
treatment <- samps$treatment

# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = treatment, ncode=1)  # Takes a while with large datasets
genotypes


## Bootstrap samples to get number of private alleles standardised by sample size

# Set up results dataframe to append to
privals <- as.data.frame(matrix(nrow=1,ncol=2))

# Add columns to data
geno.types$popns <- treatment
geno.types$sample <- rownames(geno.types)

# Run loop
library(poppr)
library(dplyr)
library(adegenet)

for(i in 1:100){
  
  # Randomly sample 100 individuals from each treatment
  df <- geno.types %>% group_by(popns) %>% sample_n(size = 100, replace=FALSE)
  df <- as.data.frame(df)
  rownames(df) <- df$sample
  df$sample <- NULL
  popns <- df$popns
  df$popns <- NULL
  
  ## Convert to genind object
  genotypes <- df2genind(df, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
  
  # Set strata
  strata(genotypes) <- data.frame(x = pop(genotypes))
  
  # Count private alleles with poppr
  priv <- private_alleles(genotypes, count.alleles=FALSE, level="population", report="table")
  priv <- rowSums(priv)
  
  # Add results to output
  colnames(privals) <- names(priv)
  privals[i,] <- priv
  
}

# Get mean values
privmeans <- as.data.frame(colMeans(privals))
privmeans

# Obtain standard errors for each popn
library("plotrix")
privmeans$se <- apply(privals,2,function(x){std.error(x)})
colnames(privmeans) <- c("mean","se")
privmeans <- round(privmeans,2)

# Output to csv
write.csv(privmeans,"output/PrivateAllelesTreatment010921.csv",row.names=TRUE)

# # Check sig
# library(reshape2)
# dfprivs <- melt(privals)
# mod2 <- glm(value ~ variable, data=dfprivs)
# car::Anova(mod2,test.statistic="F")
# 
# # Check contrasts (Tukey multiple comparison)
# library(emmeans)
# marginal = lsmeans(mod2, ~ variable)
# pairs(marginal, adjust="tukey")
# # Done











#############
# ANGSD SFS #
#############

sfs<-scan("output/grootew.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/artrange.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/kapalga.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/eastal.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/kingleo.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/mareeba.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/mornington.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/marchinbar.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/outerdarwin.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/weipa.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/astell.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/bachsten.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/kimberley.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/topend.sfs")
sfs <- sfs[1:104]
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/pilbara.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/queensland.sfs")
barplot(sfs[-1])
sum(sfs)
sfs<-scan("output/grootefull.sfs")
barplot(sfs[-1])
sum(sfs)




################
# Plot results #
################

groote <- read.table("E:/software/stairway_plot_v2.1.1/grootefull_sfs/grootefull.final.summary", header=TRUE)
queensland <- read.table("E:/software/stairway_plot_v2.1.1/queensland_sfs/queensland.final.summary", header=TRUE)
pilbara <- read.table("E:/software/stairway_plot_v2.1.1/pilbara_sfs/pilbara.final.summary", header=TRUE)
kimberley <- read.table("E:/software/stairway_plot_v2.1.1/kimberley_sfs/kimberley.final.summary", header=TRUE)
topend <- read.table("E:/software/stairway_plot_v2.1.1/topend_sfs/topend.final.summary", header=TRUE)

# groote = "#56B4E9"
# queensland = "#009E73"
# kimberley = "#E69F00"
# topend = "#F0E442"
# pilbara = "#FF0099"

# Merge into one dataframe with population label column
queensland$Population <- "Queensland"
topend$Population <- "Top End"
groote$Population <- "Groote Eylandt"
kimberley$Population <- "Kimberley"
pilbara$Population <- "Pilbara"
allqull <- rbind(queensland,topend,groote,kimberley,pilbara)

# Make facet-wrapped plot
library(ggplot2)
library(scales)
allqullmap <- ggplot(data=allqull, aes(x=year, y=Ne_median), colour=Population) +
  geom_line(stat="identity") +
  geom_ribbon(aes(x=year,ymin=Ne_2.5.,ymax=Ne_97.5., fill=Population), linetype = 2, alpha=0.4) +
  scale_fill_manual(values=c("#56B4E9","#E69F00","#FF0099","#009E73","#F0E442")) +
  facet_wrap(~ Population, nrow=5, ncol=1, scales="fixed") +
  xlab("Years before present") +
  ylab(expression(paste("Effective population size (",italic(N)*{}[e],")"))) +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.text=element_text(size = 13, color="black")) +
  scale_y_log10(limits=c(3500,12000000), expand=c(0,0), breaks=c(1000,10000,100000,1000000,10000000),
                labels = trans_format("log10", math_format(10^.x))) +
  scale_x_log10(limits=c(50,600000), expand=c(0,0), breaks=c(100,1000,10000,100000),
                labels = trans_format("log10", math_format(10^.x))) +
  theme(legend.position = "none")


## Output maps to pdf
pdffilename <- paste("output/stairway_plot_regions_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=6, height=12)
allqullmap
dev.off()




### Plot results for individual populations

GrooteWest <- read.table("E:/software/stairway_plot_v2.1.1/groote_sfs/grootew.final.summary", header=TRUE)
OuterDarwin <- read.table("E:/software/stairway_plot_v2.1.1/outerdarwin_sfs/outerdarwin.final.summary", header=TRUE)
MarbleBar <- read.table("E:/software/stairway_plot_v2.1.1/marblebar_sfs/marblebar.final.summary", header=TRUE)
Millstream <- read.table("E:/software/stairway_plot_v2.1.1/millstream_sfs/millstream.final.summary", header=TRUE)
BachstenCreek <- read.table("E:/software/stairway_plot_v2.1.1/bachsten_sfs/bachsten.final.summary", header=TRUE)
MarchinbarIsland <- read.table("E:/software/stairway_plot_v2.1.1/marchinbar_sfs/marchinbar.final.summary", header=TRUE)
Weipa <- read.table("E:/software/stairway_plot_v2.1.1/weipa_sfs/weipa.final.summary", header=TRUE)
MorningtonSanctuary <- read.table("E:/software/stairway_plot_v2.1.1/mornington_sfs/mornington.final.summary", header=TRUE)
WunaaminMiliwundiWest <- read.table("E:/software/stairway_plot_v2.1.1/kingleo_sfs/kingleo.final.summary", header=TRUE)
Mareeba <- read.table("E:/software/stairway_plot_v2.1.1/mareeba_sfs/mareeba.final.summary", header=TRUE)
Kapalga <- read.table("E:/software/stairway_plot_v2.1.1/kapalga_sfs/kapalga.final.summary", header=TRUE)
EastAlligator <- read.table("E:/software/stairway_plot_v2.1.1/eastal_sfs/eastal.final.summary", header=TRUE)
ArtesianRange <- read.table("E:/software/stairway_plot_v2.1.1/artrange_sfs/artrange.final.summary", header=TRUE)
KoolanIsland <- read.table("E:/software/stairway_plot_v2.1.1/koolan_sfs/koolan.final.summary", header=TRUE)

# Merge into one dataframe with population label column
GrooteWest$Population <- "Groote West"
OuterDarwin$Population <- "Outer Darwin"
MarbleBar$Population <- "Marble Bar"
Millstream$Population <- "Millstream Chichester NP"
BachstenCreek$Population <- "Bachsten Creek"
MarchinbarIsland$Population <- "Marchinbar Island"
Weipa$Population <- "Weipa"
MorningtonSanctuary$Population <- "Mornington Sanctuary"
WunaaminMiliwundiWest$Population <- "Wunaamin Miliwundi West"
Mareeba$Population <- "Mareeba"
Kapalga$Population <- "Kapalga"
EastAlligator$Population <- "East Alligator (2003)"
ArtesianRange$Population <- "Artesian Range"
KoolanIsland$Population <- "Koolan Island"

allqull2 <- rbind(GrooteWest,OuterDarwin,MarbleBar,Millstream,BachstenCreek,MarchinbarIsland,Weipa,KoolanIsland,
                 MorningtonSanctuary,WunaaminMiliwundiWest,Mareeba,Kapalga,EastAlligator,ArtesianRange)

allqull2$Population <- factor(allqull2$Population, levels=c("Mareeba","Weipa","Groote West",
                                                            "Marchinbar Island","Kapalga","East Alligator (2003)","Outer Darwin",
                                                            "Mornington Sanctuary","Wunaamin Miliwundi West","Bachsten Creek",
                                                            "Artesian Range","Koolan Island",
                                                            "Marble Bar","Millstream Chichester NP"))

# Make facet-wrapped plot
library(ggplot2)
library(scales)
allqullmap2 <- ggplot(data=allqull2, aes(x=year, y=Ne_median)) +
  geom_ribbon(aes(x=year,ymin=Ne_2.5.,ymax=Ne_97.5., fill=Population), linetype = 2, alpha=0.4) +
  geom_line(stat="identity",size=0.4) +
  facet_wrap(~ Population, nrow=5, ncol=3, scales="fixed") +
  scale_fill_manual(values=c("#009E73","#009E73","#56B4E9",
                             "#F0E442","#F0E442","#F0E442","#F0E442",
                             "#E69F00","#E69F00","#E69F00","#E69F00","#E69F00",
                             "#FF0099","#FF0099")) +
  xlab("Years before present") +
  ylab(expression(paste("Effective population size (",italic(N)*{}[e],")"))) +
  geom_vline(xintercept=19000, linetype=2, size=1, alpha=0.3) + # Approx time of last glacial maximum
  # geom_vline(xintercept=50000, linetype=3, size=1, alpha=0.3) + # Arrival of humans to Australia
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.text=element_text(size = 13, color="black")) +
  scale_y_log10(limits=c(85,13000000), expand=c(0,0), breaks=c(1000,100000,10000000),
                labels = trans_format("log10", math_format(10^.x))) +
  scale_x_log10(limits=c(80,600000), expand=c(0,0), breaks=c(100,1000,10000,100000),
                labels = trans_format("log10", math_format(10^.x))) +
  theme(legend.position = "none")


## Output to pdf
pdffilename <- paste("output/stairway_apriori_pops_", currentDate,".pdf",sep="")
pdf(pdffilename, family="Times", width=8, height=8)
allqullmap2
dev.off()













##################################################################
## Get filenames and make table of all loci used in BEAST analysis
# read in filenames
filens <- list.files(path = "output/nexus_beast/", pattern=".nex", recursive=TRUE)

# Get into dataframe format
filens <- filens %>%
  basename() %>%
  as.list()

filens <- unlist(filens)
filens <- str_remove(filens, pattern=".nex")
filens <- as.data.frame(str_split_fixed(filens, "_", n=3))
colnames(filens) <- c("Population","Scaffold","Position")

# Output to csv
write.csv(filens, "output/RAD_loci_BEAST.csv",quote = FALSE,row.names = FALSE)













# Commend yourself on a job well done
library(praise)
praise()
praise()
praise()
praise()
praise()


















#####################################################################################

#####################################################

## Old Code



## Nice way to get population-specific FST and FIS! (presumably no sample size correction)
library(hierfstat)
# Just calculated using the standard format of SNPs as columns, inds as rows, and 0,1,2 for genotypes 
fstfis <- fs.dosage(geno.types, pop=popns)
fist <- as.data.frame(t(fstfis$Fs))
fist
# Population specific FST measures deviation from the ancestral population






## Get number of variable loci in Marchinbar Island population (19 individuals)

mcbar <- genotypes[genotypes@pop=="Marchinbar Island"]
library(dartR)
mcbar <- gi2gl(mcbar)

x <- mcbar
na.counter <- 0
loc.list <- array(NA, nLoc(x))
mat <- as.matrix(x)
lN <- locNames(x)
for (i in 1:nLoc(x)) {
  row <- mat[, i]
  if (all(row == 0, na.rm = TRUE) | all(row == 2, na.rm = TRUE) |
      all(is.na(row))) {
    loc.list[i] <- lN[i]
    if (all(is.na(row))) {
      na.counter = na.counter + 1
    }
  }
}
loc.list <- loc.list[!is.na(loc.list)]
length(loc.list)
mcbar$n.loc-length(loc.list)











#####################
# Output to GenAlEx #
#####################

# And check significance of pairwise FST between populations

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
samps$region[samps$popn=="Astell Island 2016"] <- "English Company Islands"
samps$region[samps$popn=="Koolan Island"] <- "Kimberley"
samps$popn[samps$popn=="Mornington Station"] <- "Mornington Sanctuary"
samps$popn[samps$popn=="King Leopold West"] <- "Wunaamin Miliwundi West"
samps$popn[samps$popn=="Northwest Marble Bar"] <- "Marble Bar"
samps$popn[samps$popn=="Millstream Chichester"] <- "Millstream Chichester NP"

# Check some popn size stuff
poptab <- as.data.frame(table(samps$popn))
nrow(poptab[poptab$Freq<6,])
nrow(poptab[poptab$Freq<8,])
samps <- samps[!samps$popn %in% poptab[poptab$Freq<6,]$Var1 , ] # Remove small populations
# samps <- samps[!samps$popn == "Astell Island 2005" , ]
# samps <- samps[!samps$popn == "Darwin 2000" , ]
# samps <- samps[!samps$popn == "East Alligator 2003" , ]
# samps <- samps[!samps$popn == "King Leopold East" , ]
# samps <- samps[!samps$popn == "Pobassoo Island" , ]
length(unique(samps$popn))

# Rearrange rows so it's state -> region -> popn -> sample
library(dplyr)
samps <- samps %>% arrange(state, region, popn)

# Make sure genetic data is identical to sample metadata
identical(samps$sample,rownames(dasgen))
dasgen <- subset(dasgen, rownames(dasgen) %in% samps$sample)
dasgen <- dasgen[order(match(rownames(dasgen), samps$sample)), ]
identical(rownames(dasgen),samps$sample)
# samps$popn <- gsub(" ", "", samps$popn) # Removes spaces from population names - not necessary

# ## Use the strataG package which provides tests for gst.dbl.prime (NOT WORKING)
# library(strataG)
# # Convert to stupid gtypes format
# dasgtype <- as.data.frame(cbind(rownames(dasgen),samps$popn,dasgen[,1:ncol(dasgen)]))
# rownames(dasgtype) <- c(1:nrow(dasgtype))
# colnames(dasgtype)[1] <- "id"
# colnames(dasgtype)[2] <- "popn"
# dasgtype[1:10,1:10]
# dasgtype <- df2gtypes(dasgtype, ploidy = 1, id.col = 1, strata.col = 2, loc.col = 3)
# library(dplyr)
# overallTest(dasgtype, nrep = 3, by.locus=FALSE, stats = c("fst", "gst.dbl.prime"), quietly = FALSE, num.cores = 6) # Not working
# pairwiseTest(dasgtype, nrep = 3, by.locus=FALSE, stats = "gst.dbl.prime", quietly = FALSE, max.cores = 6) # Not working

# Convert data to right format for genind
dasgen[dasgen==2] <- 22
dasgen[dasgen==1] <- 12
dasgen[dasgen==0] <- 11
dasgen[1:10,1:10] # Check

# Subsample loci
dasgen <- dasgen[ , 1:8000]

## Convert to genind object
library(adegenet)
genotypes <- df2genind(dasgen, ploidy=2, pop=samps$popn, ncode=1)  # Takes a while with large datasets
genotypes

# # Impute/remove missingness
# library(poppr)
# genotypesnomiss <- missingno(genotypes, type = "mean", cutoff=0, quiet = FALSE, freq = FALSE)

# Convert to genalex format and output
# genind2genalex(genotypesnomiss,filename="output/quoll_genos_genalex_nomiss_20220331.csv")
genind2genalex(genotypes,filename="output/quoll_genos_genalex_w_miss_20220331.csv", overwrite=TRUE)






#################
# convert formats
# genind to genlight and back to genind

library(dartR)
genotypes <- gi2gl(genotypes, verbose=5)
genotypes <- gl.recalc.metrics(genotypes)
genotypes$other$loc.metrics <- as.data.frame(genotypes$other$loc.metrics)
genotypes <- gl.subsample.loci(genotypes, n=8180, method="random", mono.rm=TRUE, verbose=5)
# genotypes <- gl2gi(genotypes, verbose=5)

# Convert back to numeric genind
# devtools::install_github("thierrygosselin/radiator")
library(radiator)
genotypes <- genomic_converter(genotypes,output="genind")
genotypes <- genotypes$genind
genotypes










#### LOOPING THROUGH K VALUES TO OUTPUT MAPS
# ITS ROUGH BUT DAMN IS IT A NICE QUICK LOOP

## Use CreatePalette() to define color palettes.
my.colors <- c("#009E73","#FF0099","#E69F00","#F0E442","#56B4E9", "tan1", "red")
my.palette <- CreatePalette(my.colors, 10)

#FF0099 = pink
#F0E442 = yellow
#E69F00 = orange
#56B4E9 = blue
#009E73 = green

# create empty list to fill with maps for each k
mapslist <- list()

# run through k values adding each map to the list
for(i in c(1:6)){
  
  # Prepare q matrix for output
  qmat <- as.data.frame(qmatrix(tess3.obj, K = i))
  rownames(qmat) <- rownames(dasgen)
  
  # Make high resolution map
  mapslist[[i]] <- ggtess3Q(qmat, latlongs, map.polygon=histdist, col.palette=my.palette,
                            interpolation.model = FieldsKrigModel(10),
                            resolution = c(1200,1200)) +
    geom_path(data = map.polygon, aes(x = long, y = lat, group = group), colour="grey40", size=0.4) +
    xlim(113, 147) + 
    ylim(-22.5,-10) + 
    coord_equal() + 
    geom_point(data = yeahcats, aes(x = long, y = lat), size = 0.6, colour="black") + 
    xlab("Longitude") +
    ylab("Latitude") + 
    theme_bw() +
    scale_alpha(guide = "none") +
    theme(panel.grid.major.y = element_blank(),
          panel.grid.minor.y = element_blank(),
          panel.grid.major.x = element_blank(),
          panel.grid.minor.x = element_blank(),
          axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
          axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
          axis.text = element_text(size = 13, color="black")) +
    ggsn::scalebar(location="topleft",y.min=-22.5, y.max=-11, x.min=113, x.max=147, anchor=c(x=127.4,y=-22), 
                   dist=500, dist_unit="km", transform=TRUE, model='WGS84', st.dist=-0.09, st.size=4, height = 0.0365) +
    north(location = "topright", scale = 0.15, symbol = 1,
          y.min=-22.5, y.max=-10, x.min=113, x.max=147, anchor = c(x=149,y=-10.2)) +
    annotate('text', 115, -10.5, label=paste("italic(k)==", i, sep=""), parse=TRUE, hjust=1, size=5) +
    coord_quickmap()
  
  
  ## Output admixture maps to pdf
  pdffilename <- paste("output/K",i,"_admixture_maps_tess3r_", currentDate,".pdf",sep="")
  pdf(pdffilename, family="Times", width=10, height=6)
  print(mapslist[[i]])
  dev.off()
  
}













## Plotting the old heatmap using base plotting functions
# pdffilename <- paste("output/gst_plot_", currentDate,".pdf",sep="")
# pdf(pdffilename, family="Times", height=8, width=8)
# par(oma=c(3,5,0,0))
# image(1:dim, 1:dim, dst, axes = FALSE, xlab="", ylab="")
# axis(1, 1:dim, rownames(dst), cex.axis = 0.8, las=3)
# axis(2, 1:dim, rownames(dst), cex.axis = 0.8, las=1)
# text(expand.grid(1:dim, 1:dim), sprintf("%0.1f", dst), cex=0.9)
# dev.off()





## Use ggsignif for fun
library(ggsignif)
library(reshape2)
# Get raw data into correct format
dff <- allstatsproper[allstatsproper$Treatment!="Translocated",]
dff <- dff[ , c(2,6,7,11,12)]
dff <- melt(dff, id.vars = "Treatment")
dff <- dff[dff$variable=="Fis",]

# Plot
ggplot(dff, aes(x=Treatment, y=value)) +
  geom_boxplot() +
  geom_signif(comparisons = list(c("Exposed", "Naive"),
                                 c("Island", "Naive"),
                                 c("Naive", "Exposed")),
              map_signif_level = TRUE) +
  facet_wrap(~ variable, nrow=4, ncol=1, scales="free_y") +
  theme_bw() +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, colour="black"))







### Get SFS using called genotypes
# remotes::install_github("EricArcher/strataG")
library(strataG)

# Clear environment
rm(list = ls())
currentDate <- Sys.Date() # Add current date to environment for output of file names with date


# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn

# Format genotypes for strataG
genos <- as.data.frame(cbind(rownames(dasgen),popns,dasgen[,1:ncol(dasgen)]))
rownames(genos) <- c(1:nrow(genos))
genos[1:10,1:10]
colnames(genos)[1] <- "id"


library(dplyr)
genos <- filter(genos, !popns %in% c("Groote East","Groote South","Pellew Islands","Darwin 2019",
                                     "East Alligator 2017","East Alligator 2014","Kapalga 2014",
                                     "Black Mountain","Hope Vale","King Cascade","King Leopold East",
                                     "Mitchell Plateau","Mount French","Oombulgurri","Prince Regent",
                                     "Roe River Mouth","Walcott Inlet","Windjana Gorge","Dolphin Island",
                                     "Koolan Island","Litchfield National Park","Hayes Creek",
                                     "Astell Island 2005","Pobassoo Island","Darwin 2000","Oenpelli Road",
                                     "Pannawonica"))


# Loop through pops, take the SFS from each and append to df
popsfs <- as.data.frame(matrix(nrow=14,ncol=100))

# run through k values adding each map to the list
for(i in 1:length(unique(genos$popns))){
  
  sfs <- sfs(genos[genos$popns==unique(genos$popns)[i],],
             strata.col = 2,
             locus.col = 3,
             fsc.dimnames = FALSE,
             sort.strata = TRUE,
             na.action = c("filter"))
  
  popsfs[i,c(1:length(sfs$marginal[[1]]))] <- sfs$marginal[[1]]
  
}

rownames(popsfs) <- unique(genos$popns)
popsfs <- as.matrix(popsfs)


# This is an unfolded(derived)SFS. We need to get the folded version

getMaf <- function(unfolded.sfs) {
  # set the maf sfs the same as obs.sfs
  maf <- as.numeric(unfolded.sfs)
  # define the length of the SFS and half the SFS
  n <- length(maf)
  nhalf <- ceiling(n/2)
  if(n %% 2 == 0) {
    stop("This getMaf function only works for SFS with odd number of entries (for data from diploid individuals.)")
  }
  # all the entries of maf.sfs > (n/2) set to zero, where n is 2*nindividuals
  maf[(nhalf+1):n] <- 0
  # add the entry 1 to n, 2 to n-1, and so on...
  maf[1:(nhalf-1)] <- unfolded.sfs[1:(nhalf-1)]+
    unfolded.sfs[n:(nhalf+1)]
  maf
}

# Apply function to get folded SFS for each pop
groote <- getMaf(popsfs[1,c(1:33)])
astell <- getMaf(popsfs[2,c(1:27)])
mareeba <- getMaf(popsfs[3,c(1:17)])
weipa <- getMaf(popsfs[4,c(1:25)])
artrange <- getMaf(popsfs[5,c(1:39)])
bachsten <- getMaf(popsfs[6,c(1:19)])
kingleo <- getMaf(popsfs[7,c(1:25)])
mornington <- getMaf(popsfs[8,c(1:31)])
millstream <- getMaf(popsfs[9,c(1:17)])
marble <- getMaf(popsfs[10,c(1:17)])
outerdarwin <- getMaf(popsfs[11,c(1:99)])
eastal <- getMaf(popsfs[12,c(1:29)])
marchinbar <- getMaf(popsfs[13,c(1:53)])
kapalga <- getMaf(popsfs[14,c(1:43)])

# Rbind and output final sfs for each pop
install.packages("qpcR")
library(qpcR)
popsfs <- as.data.frame(qpcR:::rbind.na(groote,astell,mareeba,weipa,artrange,bachsten,kingleo,mornington,
                                        millstream,marble,outerdarwin,eastal,marchinbar,kapalga))

# Check one
barplot(outerdarwin)
barplot(marchinbar)

# Output final spreadsheet
write.csv(popsfs,"output/strataG_sfs_all_pops.csv",row.names = TRUE,quote = FALSE,na="")









###############################################
# Correlation structure in mixed effect model #
###############################################

library(nlme)
# With random effect of region
modx <- lme(He ~ Treatment,
            data = hets,
            random = ~ 1 | Region,
            correlation = corGaus(form = ~ Latitude + Longitude, metric="euclidean", fixed=FALSE))








##########################
# Output to Ne Estimator #
##########################

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
popns <- samps$popn

geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Subset to popn of interest
unique(popns)
genotypesWeipa <- genotypes[genotypes@pop==unique(popns)[12],]
genotypesMornington <- genotypes[genotypes@pop==unique(popns)[19],]
genotypesMarchinbar <- genotypes[genotypes@pop==unique(popns)[35],]

genotypesEARecent <- genotypes[genotypes@pop==unique(popns)[2],]
genotypesEAHist <- genotypes[genotypes@pop==unique(popns)[3],]


# remotes::install_github("romunov/zvau")
library(zvau)
writeGenPop(genotypesWeipa, "output/neestimator/genotypesWeipa","genotypesWeipa")
writeGenPop(genotypesMornington, "output/neestimator/genotypesMornington","genotypesMornington")
writeGenPop(genotypesMarchinbar, "output/neestimator/genotypesMarchinbar","genotypesMarchinbar")








#################
# Same for GONE #
#################

## Check that the mapping files are the same between popns
mapfile1 <- read.table(file="output/snep/artrange2013.map", header=FALSE)
mapfile2 <- read.table(file="output/snep/weipa.map", header=FALSE)
identical(mapfile1$V4,mapfile2$V4)

## Get file formatting of .ped files correct for the stupid program to run properly
files <- list.files("output/gone/", pattern = "\\.ped$", full.names=TRUE)

for(i in files){
  pedfile <- read.table(file=i, header=FALSE, colClasses = c("character"))
  pedfile$V1 <- 1
  pedfile$V5 <- 1
  pedfile <- pedfile[,1:82126]
  write.table(pedfile, i, row.names=FALSE, col.names=FALSE, sep=" ", quote=FALSE)
}


# Need to recode each ped file to unix lf format (i used notebook++) before transferring to cloud computer



## Read in results
# WA
bachsten <- read.table("output/gone/Output_Ne_bachsten", header=TRUE)
bachsten$Generation <- bachsten$Generation+4
bachsten$pop <- "Bachsten Creek"
artrange <- read.table("output/gone/Output_Ne_artrange2013", header=TRUE)
artrange$pop <- "Artesian Range"
artrange$Generation <- artrange$Generation+5
millstream <- read.table("output/gone/Output_Ne_millstream2014", header=TRUE)
millstream$pop <- "Millstream Chichester NP"
millstream$Generation <- millstream$Generation+4
koolan <- read.table("output/gone/Output_Ne_koolan2006", header=TRUE)
koolan$pop <- "Koolan Island"
koolan$Generation <- koolan$Generation+12
mornington <- read.table("output/gone/Output_Ne_mornington2012", header=TRUE)
mornington$pop <- "Mornington Station"
koolan$Generation <- koolan$Generation+6
wapops <- rbind(bachsten,artrange,millstream,koolan,mornington)


# NT
eastallig2003 <- read.table("output/gone/Output_Ne_eastalligator", header=TRUE)
eastallig2003$Generation <- eastallig2003$Generation+15
eastallig2003$pop <- "East Alligator 2003"
eastallig2017 <- read.table("output/gone/Output_Ne_eastallig2017", header=TRUE)
eastallig2017$Generation <- eastallig2017$Generation+1
eastallig2017$pop <- "East Alligator 2017"
kapalga <- read.table("output/gone/Output_Ne_kapalga94", header=TRUE)
kapalga$Generation <- kapalga$Generation+24
kapalga$pop <- "Kapalga"
outdar <- read.table("output/gone/Output_Ne_outdar2003", header=TRUE)
outdar$Generation <- outdar$Generation+15
outdar$pop <- "Outer Darwin"
groote <- read.table("output/gone/Output_Ne_grootew2018", header=TRUE)
groote$Generation <- groote$Generation
groote$pop <- "Groote Eylandt"
ntpops <- rbind(eastallig2003,eastallig2017,kapalga,outdar,groote)


# Queensland
maree <- read.table("output/gone/Output_Ne_mareeba", header=TRUE)
maree$Generation <- maree$Generation+3
maree$pop <- "Mareeba"
hopevale <- read.table("output/gone/Output_Ne_hopevale2016", header=TRUE)
hopevale$Generation <- hopevale$Generation+2
hopevale$pop <- "Hope Vale"
blackmtn <- read.table("output/gone/Output_Ne_blackmtn2016", header=TRUE)
blackmtn$Generation <- blackmtn$Generation+2
blackmtn$pop <- "Black Mountain"
weipa <- read.table("output/gone/Output_Ne_weipa", header=TRUE)
weipa$Generation <- weipa$Generation
weipa$pop <- "Weipa"
# bmhp <- read.table("output/gone/Output_Ne_bmhp2016", header=TRUE)
# bmhp$Generation <- bmhp$Generation+2
# bmhp$pop <- "BlackMtn HopeVale"
qldpops <- rbind(maree,hopevale,blackmtn,weipa)

# Combine all for facet plotting
wapops$State <- "Western Australia"
ntpops$State <- "Northern Territory"
qldpops$State <- "Queensland"
allstates <- rbind(wapops,ntpops,qldpops)

# Plot
library(ggplot2)
library(viridis)
ggplot(data=allstates, aes(Generation,Geometric_mean, colour=pop)) +
  geom_point(size=0.8) +
  geom_line(size=1) +
  facet_wrap(~ State, nrow=3, ncol=1, scales="fixed") +
  theme_bw() +
  xlab("Generations ago") +
  ylab("Ne") + 
  scale_alpha(guide = "none") +
  labs(colour = "Population") +
  theme(panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank(),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank(),
        axis.title.y=element_text(size=13, margin=margin(0,16,0,0)),
        axis.title.x=element_text(size=13, margin=margin(16,0,0,0)),
        axis.text = element_text(size = 13, color="black")) +
  scale_x_continuous(limits=c(0,100),expand = c(0, 0)) +
  scale_y_continuous(limits=c(0,3000),expand = c(0, 0)) +
  expand_limits(y = 90000000)


# Output to PDF
library(gridExtra)
pdffilename <- paste("output/GONE_Ne_history_", currentDate,".pdf",sep=" ")
pdf(pdffilename, family="Times", width=6, height=6)
gone_plot
dev.off()










### Output to DIYABC

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221_sex.csv",header=TRUE,row.names=NULL,na.strings = c("NA",""))
samps <- subset(samps,samps$sample %in% rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
identical(samps$sample,rownames(dasgen))
popns <- samps$popn
geno.types <- dasgen

# Convert data to right format for DIYABC
library(dplyr)
library(tidyr)

geno.types[geno.types==2] <- 0
geno.types[geno.types==1] <- 1
geno.types[geno.types==0] <- 2
geno.types[1:8,1:5] # Check
geno.types$IND <- rownames(geno.types)
geno.types$SEX <- samps$sex
geno.types$POP <- samps$popn
geno.types <- mutate(geno.types, across(everything(), ~replace_na(.x, 9))) # Replace missing values with 9
geno.types <- geno.types[,c(ncol(geno.types)-2,ncol(geno.types)-1,ncol(geno.types),1:(ncol(geno.types)-3))]  # Rearrange columns
geno.types <- geno.types %>% arrange(POP)
geno.types <- as.data.frame(t(geno.types))
geno.types$header <- c("IND","SEX","POP",rep("A",(nrow(geno.types)-3)))
geno.types <- as.data.frame(t(geno.types))
geno.types[1:8,1:5] # Check

# Remove small populations
smallpops <- as.data.frame(table(popns))[as.data.frame(table(popns))$Freq<6,]
smallpops <- as.character(smallpops$popns)
for(i in smallpops){
  geno.types <- geno.types[geno.types$POP!=i,]
}

# Convert pop names to single words
geno.types$POP[geno.types$POP=="Artesian Range"] <- "ArtRange"
geno.types$POP[geno.types$POP=="Astell Island 2005"] <- "Astell05"
geno.types$POP[geno.types$POP=="Astell Island 2016"] <- "Astell16"
geno.types$POP[geno.types$POP=="Bachsten Creek"] <- "Bachsten"
geno.types$POP[geno.types$POP=="Black Mountain"] <- "BlackMtn"
geno.types$POP[geno.types$POP=="Darwin 2000"] <- "Darwin2000"
geno.types$POP[geno.types$POP=="East Alligator 2003"] <- "EastAll03"
geno.types$POP[geno.types$POP=="East Alligator 2014"] <- "EastAll14"
geno.types$POP[geno.types$POP=="East Alligator 2017"] <- "EastAll17"
geno.types$POP[geno.types$POP=="Groote West"] <- "Groote"
geno.types$POP[geno.types$POP=="Hope Vale"] <- "HopeVale"
geno.types$POP[geno.types$POP=="Kapalga 1994"] <- "Kapalga"
geno.types$POP[geno.types$POP=="King Leopold East"] <- "KingLeoEast"
geno.types$POP[geno.types$POP=="King Leopold West"] <- "KingLeoWest"
geno.types$POP[geno.types$POP=="Koolan Island"] <- "KoolanIs"
geno.types$POP[geno.types$POP=="Marchinbar Island"] <- "MarchinbarIs"
geno.types$POP[geno.types$POP=="Millstream Chichester"] <- "Millstream"
geno.types$POP[geno.types$POP=="Mornington Station"] <- "Mornington"
geno.types$POP[geno.types$POP=="Northwest Marble Bar"] <- "MarbleBar"
geno.types$POP[geno.types$POP=="Outer Darwin"] <- "OuterDarwin"
geno.types$POP[geno.types$POP=="Pobassoo Island"] <- "PobassooIs"



# Output to text file for DIYABC
write.table(x=geno.types,file="output/dasy_genos_DIYABC", quote=FALSE, sep=" ",col.names = TRUE, row.names=FALSE)
# Needs slight editing in notepad++










########################################################################################
### Measuring inbreeding in all pops using the average individual inbreeding coefficient

library(adegenet)

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_040322.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
samps$region[samps$popn=="Astell Island 2016"] <- "English Company Islands"
samps$region[samps$popn=="Koolan Island"] <- "Kimberley"
samps$popn[samps$popn=="Mornington Station"] <- "Mornington Sanctuary"
samps$popn[samps$popn=="King Leopold West"] <- "Wunaamin Miliwundi West"
samps$popn[samps$popn=="Northwest Marble Bar"] <- "Marble Bar"
samps$popn[samps$popn=="Millstream Chichester"] <- "Millstream Chichester NP"

# Check some popn size stuff
poptab <- as.data.frame(table(samps$popn))
nrow(poptab[poptab$Freq<6,])
nrow(poptab[poptab$Freq<8,])
samps <- samps[!samps$popn %in% poptab[poptab$Freq<6,]$Var1 , ] # Remove small populations
# samps <- samps[!samps$popn == "Astell Island 2005" , ]
# samps <- samps[!samps$popn == "Darwin 2000" , ]
# samps <- samps[!samps$popn == "East Alligator 2003" , ]
# samps <- samps[!samps$popn == "King Leopold East" , ]
# samps <- samps[!samps$popn == "Pobassoo Island" , ]
length(unique(samps$popn))

# Rearrange rows so it's state -> region -> popn -> sample
library(dplyr)
samps <- samps %>% arrange(state, region, popn)

# Make sure genetic data is identical to sample metadata
identical(samps$sample,rownames(dasgen))
dasgen <- subset(dasgen, rownames(dasgen) %in% samps$sample)
dasgen <- dasgen[order(match(rownames(dasgen), samps$sample)), ]
identical(rownames(dasgen),samps$sample)
# samps$popn <- gsub(" ", "", samps$popn) # Removes spaces from population names - not necessary

popns <- samps$popn

# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Make population groups
unique(genotypes@pop)
sort(table(genotypes@pop))

genosAI05 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[1],]@tab)
genosAI16 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[2],]@tab)
genosPobass <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[3],]@tab)
genosGrooteW <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[4],]@tab)
genosDar2000 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[5],]@tab)
genosEA2003 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[6],]@tab)
genosEA2014 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[7],]@tab)
genosEA2017 <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[8],]@tab)
genosKapa <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[9],]@tab)
genosOutDar <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[10],]@tab)
genosMarch <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[11],]@tab)
genosBlack <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[12],]@tab)
genosHope <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[13],]@tab)
genosMareeba <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[14],]@tab)
genosWeip <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[15],]@tab)
genosArtR <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[16],]@tab)
genosBach <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[17],]@tab)
genosKingLE <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[18],]@tab)
genosKoolan <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[19],]@tab)
genosMornin <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[20],]@tab)
genosKingLW <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[21],]@tab)
genosMarBar <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[22],]@tab)
genosMillChist <- rownames(genotypes[genotypes@pop==unique(genotypes@pop)[23],]@tab)

poops <- list(genosAI05,genosAI16,genosPobass,genosGrooteW,genosDar2000,
              genosEA2003,genosEA2014,genosEA2017,genosKapa,genosOutDar,
              genosMarch,genosBlack,genosHope,genosMareeba,genosWeip,
              genosArtR,genosBach,genosKingLE,genosKoolan,genosMornin,
              genosKingLW,genosMarBar,genosMillChist)


# Run loop using the Goudet estimator - just runs off the subsetted dasgen df (instead of stupid gds format)
relatedness <- as.data.frame(matrix(nrow=3,ncol=3))
colnames(relatedness) <- c("ind1","ind2","relatedness")

# Make list to populate with results
N <- 23
kinships <- vector("list", N)

library(otuSummary)
for(i in 1:length(poops)){
  sampleids <- poops[[i]]
  rels <- subset(dasgen,rownames(dasgen) %in% sampleids)
  rels <- beta.dosage(rels,inb=FALSE,Mb=FALSE)
  rels <- matrixConvert(rels, colname = c("ind1", "ind2", "relatedness"))
  kinships[[i]] <- as.data.frame(rels)
}

# Get mean kinship of all pairs within each population
kins <- lapply(kinships, function(x) {
  mean(x$relatedness)
})

kins















##############################
# Convert to NEXUS for SNAPP #
##############################

## Clear working environment
rm(list = ls())
currentDate <- Sys.Date()

# Read in filtered dataset
dasgen <- read.csv("output/dasy_genos_010321.csv", header=TRUE, row.names=1, check.names=FALSE)
dasgen[1:10,1:10]

# Get population names
samps <- read.csv(file="output/sampinfo250221.csv",header=TRUE,row.names=NULL)
samps <- subset(samps,samps$sample %in% rownames(dasgen))
identical(samps$sample,rownames(dasgen))
samps$date <- format(as.Date(samps$date, format="%d/%m/%Y"),"%Y-%m-%d")
samps$popn[samps$popn=="Astell Island" & samps$date<2010] <- "Astell Island 2005"
samps$popn[samps$popn=="Astell Island" & samps$date>2010] <- "Astell Island 2016"
samps$popn[samps$popn=="Kapalga" & samps$date<2010] <- "Kapalga 1994"
samps$popn[samps$popn=="Kapalga" & samps$date>2010] <- "Kapalga 2014"
samps$popn[samps$popn=="East Alligator" & samps$date<2010] <- "East Alligator 2003"
samps$popn[samps$popn=="East Alligator" & samps$date>2015] <- "East Alligator 2017"
samps$popn[samps$popn=="East Alligator" & samps$date<2015] <- "East Alligator 2014"
samps$popn[samps$popn=="Darwin" & samps$date>2017] <- "Darwin 2019"
samps$popn[samps$popn=="Darwin" & samps$date<2017] <- "Darwin 2000"
samps$popn[samps$popn=="Darwin" & is.na(samps$date)] <- "Darwin 2000"
popns <- samps$popn
# write.csv(samps, "output/sampinfo_LR.csv",row.names=FALSE)

# Change name to avoid filtering if you mess up somewhere
geno.types <- dasgen

# Convert data to right format for genind
geno.types[geno.types==2] <- 22
geno.types[geno.types==1] <- 12
geno.types[geno.types==0] <- 11
geno.types[1:8,1:5] # Check

## Convert to genind object
library(adegenet)
genotypes <- df2genind(geno.types, ploidy = 2, ind.names = NULL, pop = popns, ncode=1)  # Takes a while with large datasets
genotypes

# Remove small populations
smallpops <- as.data.frame(table(popns))[as.data.frame(table(popns))$Freq<6,]
smallpops <- as.character(smallpops$popns)
for(i in smallpops){
  genotypes <- genotypes[genotypes@pop!=i,]
}

# See remaining pops
unique(genotypes@pop)


# Extract pops of interest
weipa <- genotypes[genotypes@pop=="Weipa",]

# Convert to genlight
weipa <- gi2gl(weipa)

# Output as NEXUS file for SNAPP
library(dartR)
gl2snapp(weipa, outfile = "output/weipa_snapp.nex", outpath = getwd(), verbose = NULL)










#### Old spatial model

# Using the spaMM package. Not sure if this is 100% correct method..
library(spaMM)
mod1 <- spaMM::fitme(Pi ~ Treatment + Matern(1|Latitude+Longitude %in% Region), data=hets) # with random effect of region
mod2 <- spaMM::fitme(Pi ~ Treatment + Matern(1|Latitude+Longitude), data=hets) # without random effect of region
summary(mod1)

# Investigate correlation of genomic parameter/metric by geographic distance
dd <- dist(hets[,c("Latitude","Longitude")])
mm <- MaternCorr(dd, nu = 16.666, rho = 1.96456)
plot(as.numeric(dd), as.numeric(mm), xlab = "Distance between pairs of location [in decimal degrees??]", ylab = "Estimated correlation")










###################################
# Filter out SNPs under selection #
###################################

## Use OutFlank method of (Whitlock and Lotterhos, 2015) to find loci showing
# departure from the null distribution of FST.
# This often requires, for a two-allele locus, "loci with heterozygosity greater
# than 0.1 in the whole population", because low het loci aren't representative.

library(devtools)
# Install required package qvalue if not installed
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install("qvalue")
library(qvalue)

# Install OutFLANK if not installed
# if (!("OutFLANK" %in% installed.packages())){install_github("whitlock/OutFLANK")}
library(OutFLANK)

# OutFLANK needs SNPs as columns, individuals as rows, genotypes as 0,1,2
identical(samps$sample,rownames(dasgen)) # Sample names match the order of genotypes
dasgen[1:10,1:10]

# 'NA' values need to be '9'
dasgen[is.na(dasgen)] <- 9
dasgen[1:10,1:10]

# Get the FST values for all loci
FstDataFrame <- MakeDiploidFSTMat(dasgen,locusNames=colnames(dasgen),popNames=samps$popn)
head(FstDataFrame)
nrow(FstDataFrame[FstDataFrame$He>0.025,]) # 4342 SNPs with He > 0.025

# Check some plots
library(ggplot2)
ggplot(data=FstDataFrame, aes(x=FST, y=FSTNoCorr)) +
  geom_point() +
  scale_x_continuous(expand=c(0,0)) +
  scale_y_continuous(expand=c(0,0)) +
  theme_bw()

plot(FstDataFrame$He, FstDataFrame$FSTNoCorr, pch=20, col="grey")
hist(FstDataFrame$FSTNoCorr, breaks=seq(0,1.01, by=0.003), xlim=c(0,0.9))
hist(FstDataFrame$FSTNoCorr[FstDataFrame$He>0.025], breaks=seq(0,1.01, by=0.003), xlim=c(0,0.9)) # Looks pretty good
hist(FstDataFrame$FSTNoCorr[FstDataFrame$He>0.05], breaks=seq(0,1.01, by=0.003), xlim=c(0,0.9))

summary(FstDataFrame$FST)

# Neat chi-squared distribution is what we're after, should have a long right tail
out1 <- OutFLANK(FstDataFrame, NumberOfSamples=36, LeftTrimFraction = 0.05,
                 RightTrimFraction = 0.05, Hmin = 0.025, qthreshold = 0.05)

#### NEED TO DEAL WITH THE MISSING DATA ISSUE

# Visualize results
OutFLANKResultsPlotter(out1, withOutliers = TRUE,
                       NoCorr = TRUE, Hmin = 0.025, binwidth = 0.003, Zoom =
                         FALSE, RightZoomFraction = 0.05, titletext = NULL)





